@@ -1,13 +1,15 @@
-from collections.abc import Sequence
 import json
 import os
+import re
 import sys
 import time
-from typing import Callable, Dict, List
-import yaml
+from collections.abc import Sequence
 from pathlib import Path
-from pydoc import pipepager, pager
-import re
+from pydoc import pager, pipepager
+from typing import Callable, Dict, List
+
+import yaml
+
 import spyctl.spyctl_lib as lib
 
 yaml.Dumper.ignore_aliases = lambda *args: True
@@ -78,6 +80,37 @@ def query_yes_no(question, default="yes", ignore_yes_option=False):
             )
 
 
+def notice(notice_msg):
+    """Notify the user of something, wait for input
+
+    "notice_msg" is a string that is presented to the user.
+    """
+    if YES_OPTION:
+        return
+    prompt = " [ok] "
+    sys.stderr.write(notice_msg + prompt)
+    input()
+
+
+class CustomDumper(yaml.SafeDumper):
+
+    def represent_mapping(self, tag, mapping, flow_style=None):
+        # Flow style for the
+        if (
+            mapping
+            and lib.KEY_FIELD in mapping
+            and lib.OPERATOR_FIELD in mapping
+        ):
+            return super().represent_mapping(tag, mapping, flow_style=True)
+        return super().represent_mapping(tag, mapping, flow_style)
+
+
+def make_yaml(obj) -> str:
+    return yaml.dump(
+        obj, sort_keys=False, width=float("inf"), Dumper=CustomDumper
+    )
+
+
 def show(
     obj,
     output,
@@ -103,11 +136,11 @@ def show(
     """
     out_data = None
     if output == lib.OUTPUT_YAML:
-        out_data = yaml.dump(obj, sort_keys=False)
+        out_data = make_yaml(obj)
         if output_fn:
             output_fn += ".yaml"
-    elif output == lib.OUTPUT_JSON:
-        if ndjson:
+    elif output in [lib.OUTPUT_JSON, lib.OUTPUT_NDJSON]:
+        if ndjson or output == lib.OUTPUT_NDJSON:
             if _seq_but_not_str(obj):
                 out_data = []
                 for item in obj:
