@@ -3,7 +3,6 @@ import time
 from copy import deepcopy
 from typing import Dict, Iterable, List, Optional, Union
 
-import spyctl.api as api
 import spyctl.config.configs as cfgs
 import spyctl.spyctl_lib as lib
 
@@ -30,30 +29,12 @@ POD_LABEL_TGT_FIELDS = [[*POD_SEL_TGT]]
 
 def filter_clusters(
     clusters_data: List[Dict],
-    namespaces_data=None,
-    machines_data=None,
-    pods_data=None,
-    cgroups_data=None,
-    containers_data=None,
     **filters,
 ):
-    def namespace_filter(data, filt):
-        if namespaces_data is None:
-            ctx = cfgs.get_current_context()
-            namespaces = api.get_namespaces(
-                *ctx.get_api_data(),
-                data,
-                DEFAULT_FILTER_TIME,
-            )
-            namespaces = filter_obj(namespaces, ["namespaces"], filt)
-            cluster_uids = [ns["cluster_uid"] for ns in namespaces]
-            return filter_obj(data, ["uid"], cluster_uids)
-
     filter_set = {
         cfgs.CLUSTER_FIELD: lambda data, filt: filter_obj(
             data, CLUSTERS_TGT_FIELDS, filt
         ),
-        cfgs.NAMESPACE_FIELD: namespace_filter,
     }
     clusters_data = use_filters(clusters_data, filter_set, filters)
     return clusters_data
@@ -95,13 +76,20 @@ def filter_namespaces(
     return namespaces_data
 
 
-def filter_machines(
-    machines_data: List[Dict],
+def filter_spydertraces(
+    spydertraces_data: List[Dict],
     clusters_data=None,
-    namespaces_data=None,
+    machines_data=None,
     pods_data=None,
     cgroups_data=None,
     containers_data=None,
+    **filters,
+):
+    return spydertraces_data
+
+
+def filter_sources(
+    sources_data: List[Dict],
     use_context_filters=True,
     **filters,
 ):
@@ -110,10 +98,10 @@ def filter_machines(
             data, MACHINES_TGT_FIELDS, filt
         ),
     }
-    machines_data = use_filters(
-        machines_data, filter_set, filters, use_context_filters
+    sources_data = use_filters(
+        sources_data, filter_set, filters, use_context_filters
     )
-    return machines_data
+    return sources_data
 
 
 def filter_nodes(
@@ -296,19 +284,6 @@ def filter_fingerprints(
     return fingerprint_data
 
 
-def filter_fprint_groups(
-    fprint_grp_data: List[Dict],
-    namespaces_data=None,
-    clusters_data=None,
-    machines_data=None,
-    pods_data=None,
-    cgroups_data=None,
-    containers_data=None,
-    **filters,
-):
-    return fprint_grp_data
-
-
 def filter_policies(
     policy_data: List[Dict],
     namespaces_data=None,
@@ -353,6 +328,19 @@ def filter_processes(
     return processes
 
 
+def filter_containers(
+    containers: List[Dict],
+    namespaces_data=None,
+    clusters_data=None,
+    machines_data=None,
+    pods_data=None,
+    cgroups_data=None,
+    containers_data=None,
+    **filters,
+):
+    return containers
+
+
 def filter_connections(
     connections: List[Dict],
     namespaces_data=None,
@@ -436,19 +424,28 @@ def match_filters(
     return False
 
 
+def filter_agents(
+    agents: List[Dict],
+    namespaces_data=None,
+    clusters_data=None,
+    machines_data=None,
+    pods_data=None,
+    cgroups_data=None,
+    containers_data=None,
+    **filters,
+):
+    return agents
+
+
 def get_field_value(
     field: Union[str, List[str]], obj: Dict
 ) -> Optional[Union[str, Iterable]]:
     value = obj
     if isinstance(field, str):
-        key = field
+        field = field.split(".")
+    keys = field
+    for key in keys:
         value = value.get(key)
         if value is None:
             return None
-    else:
-        keys = field
-        for key in keys:
-            value = value.get(key)
-            if value is None:
-                return None
     return value
