@@ -3,9 +3,11 @@ filtering.
 """
 
 from copy import deepcopy
-from typing import Dict, List, Tuple, Union
+from typing import Dict, List, Tuple, Union, Iterable
 
-import spyctl.api as api
+from spyctl.api.sources import get_sources
+from spyctl.api.policies import get_policies
+from spyctl.api.clusters import get_clusters
 import spyctl.config.configs as cfg
 import spyctl.filter_resource as filt
 import spyctl.spyctl_lib as lib
@@ -32,7 +34,7 @@ POLICIES_CACHE = None
 
 def get_filtered_cluids(**filters) -> List[str]:
     ctx = cfg.get_current_context()
-    clusters = api.get_clusters(*ctx.get_api_data())
+    clusters = get_clusters(*ctx.get_api_data())
     clusters = filt.filter_clusters(clusters, **filters)
     cluids = [c["uid"] for c in clusters]
     return cluids
@@ -40,7 +42,7 @@ def get_filtered_cluids(**filters) -> List[str]:
 
 def get_filtered_muids(**filters) -> List[str]:
     ctx = cfg.get_current_context()
-    sources = api.get_sources(*ctx.get_api_data())
+    sources = get_sources(*ctx.get_api_data())
     sources = filt.filter_sources(sources, **filters)
     muids = [s["uid"] for s in sources]
     return muids
@@ -50,7 +52,7 @@ def get_filtered_pol_uids(**filters) -> List[str]:
     global POLICIES_CACHE
     ctx = cfg.get_current_context()
     if not POLICIES_CACHE:
-        policies = api.get_policies(*ctx.get_api_data())
+        policies = get_policies(*ctx.get_api_data())
         POLICIES_CACHE = policies
     else:
         policies = POLICIES_CACHE
@@ -73,15 +75,62 @@ def get_filtered_pol_uids(**filters) -> List[str]:
     return policy_uids
 
 
+def get_default_time_window(resource: str) -> str:
+    if resource == lib.FINGERPRINTS_RESOURCE:
+        return "90m"
+    return "24h"
+
+
+def add_and_to_pipeline_filter(pipeline: List[Dict], filter: Dict):
+    for item in pipeline:
+        if "filter" in item:
+            filter_clause = item["filter"]
+            if "and" not in filter_clause:
+                return
+            filter_clause["and"].append(filter)
+            return
+
+
+def generate_not_clause(properties: Iterable, values: Iterable) -> Dict:
+    not_clause = {"not": {"and": []}}
+    for property, value in zip(properties, values):
+        not_clause["not"]["and"].append(generate_filter_value(property, value))
+    return not_clause
+
+
+def generate_filter_value(property, value: str) -> Dict:
+    if isinstance(value, int) or isinstance(value, float):
+        rv = {"property": property, "equals": value}
+    elif "*" in value or "?" in value:
+        value = lib.simple_glob_to_regex(value)
+        rv = {"property": property, "re_match": value}
+    else:
+        rv = {"property": property, "equals": value}
+    return rv
+
+
 class API_Filter:
-    property_map = (
-        {}
-    )  # property -> field name on object (. notation for nested fields)
-    name_or_uid_props: []  # properties in the property_map that are related to name_or_id filtering
+    # property -> field name on object
+    # (. notation for nested fields)
+    property_map = {}
+
+    # for not equals
+    # property -> field name on object
+    # (. notation for nested fields)
+    not_property_map = {}
+
+    # properties in the property_map that are
+    # related to name_or_id filtering
+    name_or_uid_props = []
+
+    # property -> callable that takes the filter value
+    # and turns it into something more useable.
+    # Ex. cluster name to cluster uid
     values_helper = {
         lib.CLUSTER_FIELD: get_filtered_cluids,
         lib.MACHINE_SELECTOR_FIELD: get_filtered_muids,
-    }  # property -> callable that takes the filter value and turns it into something more useable. Ex. cluster name to cluster uid
+    }
+
     source_type = SOURCE_TYPE_MUID
     alternate_source_type = None
 
@@ -110,6 +159,13 @@ class API_Filter:
             )
         return pipeline_items
 
+    @classmethod
+    def generate_name_or_uid_expr(
+        cls,
+        name_or_uid,
+    ):
+        return cls.__build_or_block(cls.name_or_uid_props, [name_or_uid])
+
     @classmethod
     def __generate_fprint_api_filters(
         cls,
@@ -126,27 +182,25 @@ class API_Filter:
                 )
             )
         for key, values in filters.items():
-            if isinstance(values, list) and len(values) > 1:
-                and_items.append(cls.__build_or_block([key], values))
+            if property := cls.__build_property(key):
+                if isinstance(values, list) and len(values) > 1:
+                    and_items.append(cls.__build_or_block([key], values))
+                else:
+                    if isinstance(values, list):
+                        value = values[0]
+                    else:
+                        value = values
+                    and_items.append(generate_filter_value(property, value))
+            elif property := cls.__build_not_property(key):
+                if isinstance(values, str):
+                    value = [values]
+                for value in values:
+                    and_items.append(
+                        {"not": generate_filter_value(property, value)}
+                    )
             else:
-                if isinstance(values, list):
-                    value = values[0]
-                else:
-                    value = values
-                property = cls.__build_property(key)
-                if not property:
-                    continue
-                if isinstance(value, int) or isinstance(value, float):
-                    and_items.append({"property": property, "equals": value})
-                elif "*" in value or "?" in value:
-                    value = lib.simple_glob_to_regex(value)
-                    and_items.append({"property": property, "re_match": value})
-                else:
-                    and_items.append({"property": property, "equals": value})
-        if len(and_items) > 1:
-            rv = {"filter": {"and": and_items}}
-        else:
-            rv = {"filter": and_items[0]}
+                continue
+        rv = {"filter": {"and": and_items}}
         return rv
 
     @classmethod
@@ -176,11 +230,35 @@ class API_Filter:
         return cls.property_map[key]
 
     @classmethod
-    def build_sources_and_filters(cls, **filters) -> Tuple[List[str], Dict]:
+    def __build_not_property(cls, key: str):
+        return cls.not_property_map[key]
+
+    @classmethod
+    def build_sources_and_filters(
+        cls, use_property_fields=False, **filters
+    ) -> Tuple[List[str], Dict]:
+        """
+        Builds and returns a tuple containing the sources
+        and filters based on the given parameters.
+
+        Args:
+            use_property_fields (bool):
+            Flag indicating whether to use property fields in the filters.
+            False when using the filters to build a pipeline.
+            True when using the filters standalone.
+            **filters: Additional filters to be applied.
+
+        Returns:
+            Tuple[List[str], Dict]: A tuple containing the sources
+            (list of strings) and filters (dictionary).
+
+        """
         ctx = cfg.get_current_context()
         ctx_filters = deepcopy(ctx.get_filters())
         sources = cls.__get_sources(ctx_filters, filters)
         filters = cls.__get_filters(ctx_filters, filters)
+        if use_property_fields:
+            filters = {cls.__build_property(k): v for k, v in filters.items()}
         return sources, filters
 
     @classmethod
@@ -271,6 +349,10 @@ class Agents(API_Filter):
         lib.AGENT_ID: "id",
         lib.AGENT_HOSTNAME: "hostname",
         lib.STATUS_FIELD: lib.STATUS_FIELD,
+        lib.CLUSTER_FIELD: lib.AGENT_CLUSTER_NAME,
+    }
+    values_helper = {
+        lib.MACHINE_SELECTOR_FIELD: get_filtered_muids,
     }
     name_or_uid_props = [lib.AGENT_ID, lib.AGENT_HOSTNAME, lib.MACHINES_FIELD]
     source_type = SOURCE_TYPE_GLOBAL
@@ -286,11 +368,6 @@ class Agents(API_Filter):
             schema, name_or_uid, latest_model, filters
         )
 
-    @classmethod
-    def generate_metrics_pipeline(cls):
-        schema = lib.EVENT_AGENT_METRICS_PREFIX
-        return super(Agents, cls).generate_pipeline(schema, latest_model=True)
-
 
 class AgentMetrics(API_Filter):
     property_map = {lib.ID_FIELD: lib.ID_FIELD}
@@ -301,7 +378,10 @@ class AgentMetrics(API_Filter):
     def generate_pipeline(
         cls, name_or_uid=None, type=None, latest_model=True, filters={}
     ) -> List:
-        schema = lib.EVENT_AGENT_METRICS_PREFIX
+        schema = (
+            f"{lib.EVENT_METRICS_PREFIX}:"
+            f"{lib.EVENT_METRIC_SUBTYPE_MAP['agent']}"
+        )
         return super(AgentMetrics, cls).generate_pipeline(
             schema, name_or_uid, latest_model, filters
         )
@@ -392,7 +472,9 @@ class Deployments(API_Filter):
         lib.ID_FIELD: lib.ID_FIELD,
         lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
         lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
-        lib.NAMESPACE_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}",
+        ),
     }
     name_or_uid_props = [lib.ID_FIELD, lib.NAME_FIELD, lib.BE_KUID_FIELD]
     source_type = SOURCE_TYPE_CLUID_BASE
@@ -412,6 +494,9 @@ class Deviations(API_Filter):
         lib.ID_FIELD: lib.ID_FIELD,
         lib.POLICIES_FIELD: lib.BE_POL_UID_FIELD,
     }
+    not_property_map = {
+        f"not_{lib.CHECKSUM_FIELD}": lib.CHECKSUM_FIELD,
+    }
     name_or_uid_props = [lib.ID_FIELD]
     source_type = SOURCE_TYPE_POL
 
@@ -419,23 +504,17 @@ class Deviations(API_Filter):
     def generate_pipeline(
         cls, name_or_uid=None, latest_model=True, filters={}
     ) -> List:
-        schema = (
-            f"{lib.EVENT_AUDIT_PREFIX}:"
-            f"{lib.EVENT_AUDIT_SUBTYPE_MAP['deviation']}"
-        )
+        schema = lib.EVENT_DEVIATION_PREFIX
         return super(Deviations, cls).generate_pipeline(
             schema, name_or_uid, latest_model, filters
         )
 
     @classmethod
-    def generate_count_pipeline(cls, filters={}):
-        schema = (
-            f"{lib.EVENT_AUDIT_PREFIX}:"
-            f"{lib.EVENT_AUDIT_SUBTYPE_MAP['deviation']}"
-        )
+    def generate_count_pipeline(cls, name_or_uid, filters={}):
+        schema = lib.EVENT_DEVIATION_PREFIX
         pipeline_items = super(Deviations, cls).generate_pipeline(
             schema,
-            None,
+            name_or_uid,
             True,
             filters,
         )
@@ -456,16 +535,27 @@ class Deviations(API_Filter):
 
 
 class Fingerprints(API_Filter):
+    # property_map = {
+    #     lib.MACHINES_FIELD: "muid",
+    #     lib.POD_FIELD: "pod_uid",
+    #     lib.CLUSTER_FIELD: "cluster_uid",
+    #     lib.NAMESPACE_FIELD: (
+    #       f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+    # ),
+    #     lib.CGROUP_FIELD: "cgroup",
+    #     lib.IMAGE_FIELD: "image",
+    #     lib.IMAGEID_FIELD: "image_id",
+    #     lib.CONTAINER_ID_FIELD: "container_id",
+    #     lib.CONTAINER_NAME_FIELD: "container_name",
+    #     lib.STATUS_FIELD: lib.STATUS_FIELD,
+    #     lib.ID_FIELD: lib.ID_FIELD,
+    # }
     property_map = {
-        lib.MACHINES_FIELD: "muid",
-        lib.POD_FIELD: "pod_uid",
         lib.CLUSTER_FIELD: "cluster_uid",
-        lib.NAMESPACE_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}",
+        lib.NAMESPACE_FIELD: lib.METADATA_NAMESPACE_FIELD,
         lib.CGROUP_FIELD: "cgroup",
         lib.IMAGE_FIELD: "image",
         lib.IMAGEID_FIELD: "image_id",
-        lib.CONTAINER_ID_FIELD: "container_id",
-        lib.CONTAINER_NAME_FIELD: "container_name",
         lib.STATUS_FIELD: lib.STATUS_FIELD,
         lib.ID_FIELD: lib.ID_FIELD,
     }
@@ -515,19 +605,20 @@ class Namespaces(API_Filter):
         lib.ID_FIELD: lib.ID_FIELD,
         lib.BE_KUID_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_UID_FIELD}",
         lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
-        lib.NAMESPACE_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
     }
-    # Namespaces aren't filtered by name at the API Level
     name_or_uid_props = [lib.ID_FIELD, lib.NAME_FIELD, lib.BE_KUID_FIELD]
     source_type = SOURCE_TYPE_CLUID_BASE
 
     @classmethod
-    def generate_pipeline(cls, latest_model=True, filters={}) -> List:
-        # Namespace objects don't exist in spyderbat's backend. Clusters
-        # track the metadata of namespaces though.
-        schema = lib.MODEL_CLUSTER_PREFIX
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_NAMESPACE_PREFIX
         return super(Namespaces, cls).generate_pipeline(
-            schema, None, latest_model, filters
+            schema, name_or_uid, latest_model, filters
         )
 
     @classmethod
@@ -543,7 +634,9 @@ class Nodes(API_Filter):
         lib.ID_FIELD: lib.ID_FIELD,
         lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
         lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
-        lib.NAMESPACE_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
     }
     name_or_uid_props = [lib.ID_FIELD, lib.NAME_FIELD, lib.BE_KUID_FIELD]
     source_type = SOURCE_TYPE_CLUID_BASE
@@ -579,7 +672,9 @@ class Pods(API_Filter):
         lib.ID_FIELD: lib.ID_FIELD,
         lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
         lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
-        lib.NAMESPACE_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
     }
     name_or_uid_props = [lib.ID_FIELD, lib.METADATA_NAME_FIELD]
     source_type = SOURCE_TYPE_CLUID_POCO
@@ -594,6 +689,116 @@ class Pods(API_Filter):
         )
 
 
+class ReplicaSet(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
+        lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
+    }
+    name_or_uid_props = [lib.ID_FIELD, lib.METADATA_NAME_FIELD]
+    source_type = SOURCE_TYPE_CLUID_BASE
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_REPLICASET_PREFIX
+        return super(ReplicaSet, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
+class Role(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
+        lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
+    }
+    name_or_uid_props = [lib.ID_FIELD, lib.METADATA_NAME_FIELD]
+    source_type = SOURCE_TYPE_CLUID_BASE
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_K8S_ROLE_PREFIX
+        return super(Role, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
+class ClusterRole(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
+        lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
+    }
+    name_or_uid_props = [lib.ID_FIELD, lib.METADATA_NAME_FIELD]
+    source_type = SOURCE_TYPE_CLUID_BASE
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_K8S_CLUSTERROLE_PREFIX
+        return super(ClusterRole, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
+class RoleBinding(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
+        lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
+    }
+    name_or_uid_props = [lib.ID_FIELD, lib.METADATA_NAME_FIELD]
+    source_type = SOURCE_TYPE_CLUID_BASE
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_ROLEBINDING_PREFIX
+        return super(RoleBinding, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
+class ClusterRoleBinding(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
+        lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
+    }
+    name_or_uid_props = [lib.ID_FIELD, lib.METADATA_NAME_FIELD]
+    source_type = SOURCE_TYPE_CLUID_BASE
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_CLUSTERROLE_BINDING_PREFIX
+        return super(ClusterRoleBinding, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
 class Processes(API_Filter):
     property_map = {
         lib.ID_FIELD: lib.ID_FIELD,
@@ -619,6 +824,28 @@ class Processes(API_Filter):
         )
 
 
+class Daemonsets(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.BE_KUID_FIELD: lib.BE_KUID_FIELD,
+        lib.NAME_FIELD: f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}",
+        lib.NAMESPACE_FIELD: (
+            f"{lib.METADATA_FIELD}.{lib.METADATA_NAMESPACE_FIELD}"
+        ),
+    }
+    name_or_uid_props = [lib.ID_FIELD, lib.NAME_FIELD, lib.BE_KUID_FIELD]
+    source_type = SOURCE_TYPE_CLUID_BASE
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = lib.MODEL_DAEMONSET_PREFIX
+        return super(Daemonsets, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
 class RedFlags(API_Filter):
     property_map = {
         lib.ID_FIELD: lib.ID_FIELD,
@@ -673,7 +900,9 @@ class Spydertraces(API_Filter):
 
 
 class SpydertraceSummaries(API_Filter):
-    property_map = {lib.ID_FIELD: lib.ID_FIELD}
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+    }
     name_or_uid_props = [lib.ID_FIELD]
     source_type = SOURCE_TYPE_MUID
 
@@ -687,12 +916,30 @@ class SpydertraceSummaries(API_Filter):
         )
 
 
+class SpydertopData(API_Filter):
+    property_map = {
+        lib.ID_FIELD: lib.ID_FIELD,
+        lib.MACHINES_FIELD: "muid",
+    }
+    name_or_uid_props = []
+    source_type = SOURCE_TYPE_MUID
+
+    @classmethod
+    def generate_pipeline(
+        cls, name_or_uid=None, latest_model=True, filters={}
+    ) -> List:
+        schema = f"{lib.EVENT_TOP_DATA_PREFIX}"
+        return super(SpydertopData, cls).generate_pipeline(
+            schema, name_or_uid, latest_model, filters
+        )
+
+
 class UID_List(API_Filter):
     @classmethod
-    def generate_pipeline(cls, uid_list: Dict):
+    def generate_pipeline(cls, uid_list: List[str]):
         pipeline_items = [{"latest_model": {}}]
         or_items = []
-        for uid in uid_list[lib.DATA_FIELD][lib.UIDS_FIELD]:
+        for uid in uid_list:
             or_items.append({"property": lib.ID_FIELD, "equals": uid})
         filter = {"filter": {"or": or_items}}
         pipeline_items.append(filter)
