@@ -6,7 +6,7 @@ from tabulate import tabulate
 
 import spyctl.config.configs as cfg
 import spyctl.spyctl_lib as lib
-import spyctl.api as api
+from spyctl.api.source_query_resources import get_connections
 
 NOT_AVAILABLE = lib.NOT_AVAILABLE
 
@@ -77,7 +77,7 @@ def conn_summary_output(
     limit_mem=False,
 ) -> str:
     groups: Dict[str, ConnectionGroup] = {}
-    for conn in api.get_connections(
+    for conn in get_connections(
         *ctx.get_api_data(),
         muids,
         time,
@@ -151,7 +151,7 @@ def _shorten_v6(ip):
     last = 0
     num = 0
     while i < len(ip):
-        if ip[i : i + 4] == "zero":
+        if ip[i : i + 4] == "zero":  # noqa
             if num == 0:
                 last = i
             num += 1
@@ -165,7 +165,7 @@ def _shorten_v6(ip):
     if ip.endswith("*"):
         len_diff += 1
     if pos != -1:
-        ip = ip[:pos] + ":" + ip[pos + most * 5 :]
+        ip = ip[:pos] + ":" + ip[pos + most * 5 :]  # noqa
         if pos == 0:
             ip = ":" + ip
     ip = ip.replace("0", "")
