@@ -2,7 +2,7 @@ from typing import Dict, List, Tuple
 
 from tabulate import tabulate
 
-import spyctl.api as api
+from spyctl.api.source_query_resources import get_machines
 import spyctl.config.configs as cfg
 import spyctl.spyctl_lib as lib
 
@@ -17,7 +17,7 @@ def machines_summary_output(
     limit_mem=False,
 ) -> str:
     data = []
-    for machine in api.get_machines(
+    for machine in get_machines(
         *ctx.get_api_data(), muids, time, pipeline, limit_mem
     ):
         data.append(__machine_summary_data(machine))
