@@ -3,10 +3,10 @@ from typing import Dict, List, Tuple
 from tabulate import tabulate
 
 import spyctl.config.configs as cfg
-import spyctl.spyctl_lib as lib
-import spyctl.api as api
-import spyctl.resources.api_filters as _af
 import spyctl.filter_resource as filt
+import spyctl.resources.api_filters as _af
+import spyctl.spyctl_lib as lib
+from spyctl.api.source_query_resources import get_namespaces
 
 SUMMARY_HEADERS = ["NAME", "LAST_SEEN_STATUS", "AGE", "CLUSTER"]
 
@@ -20,7 +20,7 @@ def namespace_summary_output(
 ) -> str:
     data = []
     field_names = _af.Namespaces.get_name_or_uid_fields()
-    for namespace in api.get_namespaces(
+    for namespace in get_namespaces(
         *ctx.get_api_data(), clusters, time, pipeline
     ):
         ns = [namespace]
