@@ -2,9 +2,9 @@ from typing import Dict, List, Tuple
 
 from tabulate import tabulate
 
-import spyctl.api as api
 import spyctl.config.configs as cfg
 import spyctl.spyctl_lib as lib
+from spyctl.api.source_query_resources import get_nodes
 
 SUMMARY_HEADERS = [
     "NAME",
@@ -24,7 +24,7 @@ def nodes_output_summary(
     limit_mem=False,
 ) -> str:
     data = []
-    for node in api.get_nodes(
+    for node in get_nodes(
         *ctx.get_api_data(), clusters, time, pipeline, limit_mem
     ):
         data.append(__node_summary_data(node))
