@@ -2,7 +2,7 @@ from typing import Dict, List, Tuple
 
 from tabulate import tabulate
 
-import spyctl.api as api
+from spyctl.api.source_query_resources import get_pods
 import spyctl.config.configs as cfg
 import spyctl.spyctl_lib as lib
 
@@ -26,7 +26,7 @@ def pods_output_summary(
     limit_mem=False,
 ) -> str:
     data = []
-    for pod in api.get_pods(
+    for pod in get_pods(
         *ctx.get_api_data(), clusters, time, pipeline, limit_mem
     ):
         data.append(__pod_summary_data(pod))
