@@ -1,36 +1,47 @@
+# pylint: disable=broad-exception-caught
+
 import json
-from typing import Dict, List, Optional, Tuple, Union, Set
+from typing import Dict, List, Optional, Set, Tuple, Union
 
 import zulu
 from tabulate import tabulate
 
-import spyctl.api as api
-import spyctl.cli as cli
+import spyctl.api.policies as api
 import spyctl.config.configs as cfg
 import spyctl.filter_resource as filt
-import spyctl.merge_lib as m_lib
+import spyctl.merge_lib.merge_object as m_obj
+import spyctl.merge_lib.merge_object_helper as _m_obj_h
+import spyctl.merge_lib.merge_schema as _ms
 import spyctl.resources.api_filters as _af
 import spyctl.resources.fingerprints as spyctl_fprints
 import spyctl.resources.resources_lib as r_lib
 import spyctl.schemas_v2 as schemas
 import spyctl.spyctl_lib as lib
+from spyctl import cli
+from spyctl.api.source_query_resources import get_fingerprints
+from spyctl.config.configs import get_current_context
+from spyctl.filter_resource import filter_obj
 
 FPRINT_KIND = spyctl_fprints.FPRINT_KIND
 GROUP_KIND = spyctl_fprints.GROUP_KIND
 BASELINE_KIND = lib.BASELINE_KIND
 POLICY_KIND = lib.POL_KIND
 
-POLICY_META_MERGE_SCHEMA = m_lib.MergeSchema(
-    lib.METADATA_FIELD,
-    merge_functions={
-        lib.METADATA_NAME_FIELD: m_lib.keep_base_value_merge,
-        lib.METADATA_TYPE_FIELD: m_lib.keep_base_value_merge,
-        lib.METADATA_UID_FIELD: m_lib.keep_base_value_merge,
-        lib.METADATA_CREATE_TIME: m_lib.keep_base_value_merge,
-        lib.LATEST_TIMESTAMP_FIELD: m_lib.greatest_value_merge,
-    },
-)
-POLICY_MERGE_SCHEMAS = [POLICY_META_MERGE_SCHEMA, m_lib.SPEC_MERGE_SCHEMA]
+POLICY_SUB_TYPES = {lib.POL_TYPE_TRACE: "suppression"}
+
+
+def get_policy_subtype(pol_type: str) -> str:
+    """
+    Returns the subtype of a policy based on its type.
+
+    Args:
+        pol_type (str): The type of the policy.
+
+    Returns:
+        str: The subtype of the policy. If the subtype is not found, it
+            returns "guardian".
+    """
+    return POLICY_SUB_TYPES.get(pol_type, "guardian")
 
 
 class InvalidPolicyError(Exception):
@@ -130,7 +141,7 @@ class Policy:
 def get_data_for_api_call(policy: Policy) -> Tuple[Optional[str], str]:
     policy = policy.as_dict()
     name = policy[lib.METADATA_FIELD][lib.METADATA_NAME_FIELD]
-    type = policy[lib.METADATA_FIELD][lib.METADATA_TYPE_FIELD]
+    pol_type = policy[lib.METADATA_FIELD][lib.METADATA_TYPE_FIELD]
     tags = policy[lib.METADATA_FIELD].get(lib.METADATA_TAGS_FIELD)
     uid = policy[lib.METADATA_FIELD].get(lib.METADATA_UID_FIELD, "")
     policy_selectors = {
@@ -142,7 +153,7 @@ def get_data_for_api_call(policy: Policy) -> Tuple[Optional[str], str]:
         lib.API_REQ_FIELD_NAME: name[:32],
         lib.API_REQ_FIELD_POLICY: json.dumps(policy),
         lib.API_REQ_FIELD_POL_SELECTORS: json.dumps(policy_selectors),
-        lib.API_REQ_FIELD_TYPE: type,
+        lib.API_REQ_FIELD_TYPE: pol_type,
         lib.API_REQ_FIELD_UID: uid,
     }
     if tags:
@@ -159,6 +170,7 @@ def create_policy(
     ctx: cfg.Context = None,
     disable_procs: str = None,
     disable_conns: str = None,
+    include_imageid: bool = False,
 ):
     input_objs = []
     if isinstance(input_data, list):
@@ -170,9 +182,9 @@ def create_policy(
         input_objs.extend(r_lib.handle_input_data(input_data, ctx))
     if len(input_objs) == 0:
         cli.err_exit("Nothing to build policy with")
-    merge_object = m_lib.MergeObject(
+    merge_object = m_obj.MergeObject(
         input_objs[0],
-        POLICY_MERGE_SCHEMAS,
+        _ms.POLICY_MERGE_SCHEMAS,
         None,
         disable_procs=disable_procs,
         disable_conns=disable_conns,
@@ -196,6 +208,10 @@ def create_policy(
     rv = policy.as_dict()
     if not schemas.valid_object(rv):
         cli.err_exit("Created policy failed validation.")
+    if not include_imageid:
+        container_selector = rv[lib.SPEC_FIELD].get(lib.CONT_SELECTOR_FIELD)
+        if container_selector:
+            container_selector.pop(lib.IMAGEID_FIELD, None)
     return rv
 
 
@@ -219,7 +235,7 @@ def matching_fingerprints_summary(
     sources, filters = _af.Fingerprints.build_sources_and_filters()
     pipeline = _af.Fingerprints.generate_pipeline(filters=filters)
     fingerprints = list(
-        api.get_fingerprints(
+        get_fingerprints(
             *ctx.get_api_data(),
             sources,
             time,
@@ -265,7 +281,12 @@ def policies_summary_output(
     time: Tuple[float, float] = None,
     get_deviations_count: bool = False,
     suppress_msg=False,
+    dev_name_or_uid=None,
+    dev_filters=None,
+    include_irrelevant=False,
 ):
+    if dev_filters is None:
+        dev_filters = {}
     output_list = []
     if get_deviations_count:
         headers = [
@@ -273,15 +294,23 @@ def policies_summary_output(
             "NAME",
             "STATUS",
             "TYPE",
+            "VERSION",
             "CREATE_TIME",
             "DEVIATIONS_(UNIQ/TOT)",
         ]
     else:
-        headers = ["UID", "NAME", "STATUS", "TYPE", "CREATE_TIME"]
+        headers = ["UID", "NAME", "STATUS", "TYPE", "VERSION", "CREATE_TIME"]
     data = []
     deviation_counts = {}
     if get_deviations_count:
-        deviation_counts = get_deviation_counts(policies, time, suppress_msg)
+        deviation_counts = get_deviation_counts(
+            policies,
+            time,
+            suppress_msg,
+            dev_name_or_uid,
+            dev_filters=dev_filters,
+            include_irrelevant=include_irrelevant,
+        )
     for policy in policies:
         data.append(
             policy_summary_data(policy, deviation_counts, get_deviations_count)
@@ -292,8 +321,10 @@ def policies_summary_output(
 
 
 def policy_summary_data(
-    policy: Dict, deviations_counts={}, get_deviations_count=False
+    policy: Dict, deviations_counts=None, get_deviations_count=False
 ):
+    if deviations_counts is None:
+        deviations_counts = {}
     uid = policy[lib.METADATA_FIELD].get(lib.METADATA_UID_FIELD)
     status = policy[lib.SPEC_FIELD].get(lib.ENABLED_FIELD, True)
     mode = policy[lib.SPEC_FIELD].get(lib.POL_MODE_FIELD, lib.POL_MODE_ENFORCE)
@@ -324,6 +355,7 @@ def policy_summary_data(
         policy[lib.METADATA_FIELD][lib.NAME_FIELD],
         status,
         policy[lib.METADATA_FIELD][lib.TYPE_FIELD],
+        policy[lib.METADATA_FIELD][lib.VERSION_FIELD],
         create_time,
     ]
     if get_deviations_count:
@@ -338,13 +370,10 @@ def policy_summary_data(
 def get_policy_by_uid(
     uid: str, policies: Optional[List[Dict]] = None
 ) -> Optional[Dict]:
-    from spyctl.api import get_policies
-    from spyctl.config.configs import get_current_context
-    from spyctl.filter_resource import filter_obj
 
     ctx = get_current_context()
     if not policies:
-        policies = get_policies(*ctx.get_api_data())
+        policies = api.get_policies(*ctx.get_api_data())
     policies = filter_obj(
         policies,
         [
@@ -358,8 +387,17 @@ def get_policy_by_uid(
 
 
 def get_deviation_counts(
-    policies: List[Dict], time: Tuple[float, float], suppress_msg=False
+    policies: List[Dict],
+    time: Tuple[float, float],
+    suppress_msg=False,
+    dev_name_or_uid=None,
+    dev_filters=None,
+    include_irrelevant=False,
 ) -> Dict:
+    import spyctl.resources.deviations as spyctl_dev
+
+    if dev_filters is None:
+        dev_filters = {}
     if not suppress_msg:
         cli.try_log(
             f"Getting policy deviations from {lib.epoch_to_zulu(time[0])} to"
@@ -367,24 +405,33 @@ def get_deviation_counts(
         )
     rv: Dict[str, List[Set, int]] = {}
     ctx = cfg.get_current_context()
-    policy_uids = [
-        policy[lib.METADATA_FIELD].get(lib.METADATA_UID_FIELD)
+    policy_uids = {
+        policy[lib.METADATA_FIELD].get(
+            lib.METADATA_UID_FIELD
+        ): _m_obj_h.get_merge_object(
+            lib.POL_KIND, policy, True, "check_deviations"
+        )
         for policy in policies
-    ]
-    pipeline = _af.Deviations.generate_count_pipeline()
-    checksums = set()
-    for count_obj in api.get_deviations(
-        *ctx.get_api_data(),
-        policy_uids,
+    }  # policy uid -> merge object
+    pipeline = _af.Deviations.generate_pipeline(
+        dev_name_or_uid, filters=dev_filters
+    )
+    for deviation in spyctl_dev.get_deviations_stream(
+        ctx,
+        list(policy_uids),
         time,
         pipeline,
+        disable_pbar_on_first=not lib.is_redirected(),
+        raw_data=True,
+        policies=policies,
+        include_irrelevant=include_irrelevant,
     ):
-        uid = count_obj["policy_uid"]
-        checksums.update(count_obj["counts"])
-        if uid not in rv:
-            rv[uid] = [set(), 0]
-        rv[uid][0].update(count_obj["counts"])
-        rv[uid][1] += sum(count_obj["counts"].values())
+        checksum = deviation[lib.CHECKSUM_FIELD]
+        pol_uid = deviation["policy_uid"]
+        if pol_uid not in rv:
+            rv[pol_uid] = [set(), 0]
+        rv[pol_uid][0].add(checksum)
+        rv[pol_uid][1] += 1
     return rv
 
 
