@@ -3,7 +3,7 @@ from typing import Dict, List, Tuple
 import zulu
 from tabulate import tabulate
 
-import spyctl.api as api
+from spyctl.api.source_query_resources import get_processes
 import spyctl.config.configs as cfg
 import spyctl.spyctl_lib as lib
 
@@ -68,7 +68,7 @@ def processes_stream_output_summary(
     limit_mem=False,
 ) -> str:
     groups: Dict[Tuple, ProcessGroup] = {}
-    for proc in api.get_processes(
+    for proc in get_processes(
         *ctx.get_api_data(), muids, time, pipeline, limit_mem
     ):
         multiple_exes, key = _key(proc)
