@@ -13,7 +13,9 @@ from fnmatch import fnmatch
 from hashlib import md5
 from pathlib import Path
 from typing import IO, Any, Dict, Iterable, List, Optional, Tuple, Union
+from urllib.parse import urlparse
 from uuid import uuid4
+from click_aliases import ClickAliasedGroup
 
 import click
 import dateutil.parser as dateparser
@@ -23,10 +25,13 @@ from click.shell_completion import CompletionItem
 
 
 class Aliases:
-    def __init__(self, aliases: Iterable[str], name, name_plural="") -> None:
+    def __init__(
+        self, aliases: Iterable[str], name, name_plural="", kind=None
+    ) -> None:
         self.name = name
         self.name_plural = name_plural
         self.aliases = set(aliases)
+        self.kind = kind
 
     def __eq__(self, __o: object) -> bool:
         plural_match = __o == self.name_plural if self.name_plural else False
@@ -45,6 +50,7 @@ ADD_COLOR = "\x1b[38;5;35m"
 SUB_COLOR = "\x1b[38;5;203m"
 COLOR_END = "\x1b[0m"
 API_CALL = False
+INTERACTIVE = False
 DEBUG = False
 LOG_VAR = []
 ERR_VAR = []
@@ -76,6 +82,20 @@ def flush_err_var() -> str:
     return rv
 
 
+# Resource Kinds
+BASELINE_KIND = "SpyderbatBaseline"
+DEVIATION_KIND = "GuardianDeviation"
+FPRINT_GROUP_KIND = "FingerprintGroup"
+FPRINT_KIND = "SpyderbatFingerprint"
+NOTIFICATION_KIND = "NotificationConfiguration"
+NOTIF_TMPL_KIND = "NotificationConfigTemplate"
+POL_KIND = "SpyderbatPolicy"
+SUP_POL_KIND_ALIAS = "SuppressionPolicy"
+TARGET_KIND = "NotificationTarget"
+UID_LIST_KIND = "UidList"
+RULESET_KIND = "SpyderbatRuleset"
+SAVED_QUERY_KIND = "SpyderbatSavedQuery"
+
 # Resource Aliases
 AGENT_RESOURCE = Aliases(
     ["agents", "agent", "ag"],
@@ -94,12 +114,13 @@ BASELINES_RESOURCE = Aliases(
     ],
     "baseline",
     "baselines",
+    kind=BASELINE_KIND,
 )
 CLUSTERS_RESOURCE = Aliases(
     ["clusters", "cluster", "clust", "clusts", "clus"], "cluster", "clusters"
 )
 CONTAINER_RESOURCE = Aliases(
-    ["container", "containers", "cont" "c"],
+    ["container", "containers", "cont", "c"],
     "container",
     "containers",
 )
@@ -122,6 +143,12 @@ CONNECTION_BUN_RESOURCE = Aliases(
     "connection-bundle",
     "connection-bundles",
 )
+CLUSTER_POLICY_RESOURCE = Aliases(
+    ["cluster-policy", "cluster-policies", "clus-pol", "cp"],
+    "cluster-policy",
+    "cluster-policies",
+    kind=POL_KIND,
+)
 DEPLOYMENTS_RESOURCE = Aliases(
     ["deployments", "deployment", "deploys", "deploy"],
     "deployment",
@@ -131,11 +158,18 @@ DEVIATIONS_RESOURCE = Aliases(
     ["deviations", "deviation", "dev"],
     "deviation",
     "deviations",
+    kind=DEVIATION_KIND,
+)
+DAEMONSET_RESOURCE = Aliases(
+    ["daemonset", "daemonsets", "daemon", "ds"],
+    "daemonset",
+    "daemonsets",
 )
 FINGERPRINT_GROUP_RESOURCE = Aliases(
     ["fingerprint-group", "fingerprint-groups", "fprint-group", "fg"],
     "fingerprint-group",
     "fingerprint-groups",
+    kind=FPRINT_GROUP_KIND,
 )
 FINGERPRINTS_RESOURCE = Aliases(
     [
@@ -151,6 +185,7 @@ FINGERPRINTS_RESOURCE = Aliases(
     ],
     "fingerprint",
     "fingerprints",
+    kind=FPRINT_KIND,
 )
 MACHINES_RESOURCE = Aliases(
     ["machines", "mach", "machs", "machine"],
@@ -162,10 +197,84 @@ NAMESPACES_RESOURCE = Aliases(
     "namespace",
     "namespaces",
 )
+NOTIFICATION_CONFIGS_RESOURCE = Aliases(
+    [
+        "notification-config",
+        "notification-configs",
+        "notification-policy",
+        "nc",
+    ],
+    "notification-config",
+    "notification-configs",
+    kind=NOTIFICATION_KIND,
+)
+NOTIFICATION_CONFIG_TEMPLATES_RESOURCE = Aliases(
+    [
+        "notification-config-template",
+        "notification-config-templates",
+        "notification-config-tmpls",
+        "notif-config-tmpl",
+        "notif-config-tmpls",
+        "nct",
+    ],
+    "notification-config-template",
+    "notification-config-templates",
+    kind=NOTIF_TMPL_KIND,
+)
+NOTIFICATION_TARGETS_RESOURCE = Aliases(
+    [
+        "target",
+        "targets",
+        "notification-target",
+        "notification-targets",
+        "nt",
+    ],
+    "notification-target",
+    "notification-targets",
+    kind=TARGET_KIND,
+)
 NODES_RESOURCE = Aliases(["nodes", "node"], "node", "nodes")
 OPSFLAGS_RESOURCE = Aliases(["opsflags", "opsflag"], "opsflag", "opsflags")
 PODS_RESOURCE = Aliases(["pods", "pod"], "pod", "pods")
 REDFLAGS_RESOURCE = Aliases(["redflags", "redflag"], "redflag", "redflags")
+CONTAINER_POL_RESOURCE = Aliases(
+    [
+        "container-policy",
+        "container-policies",
+        "cont-pol",
+    ],
+    "container-policy",
+    "container-policies",
+    kind=POL_KIND,
+)
+LINUX_SVC_RESOURCE = Aliases(
+    ["linux-service", "linux-services", "linux-svc", "linux-svcs"],
+    "linux-service",
+    "linux-services",
+)
+LINUX_SVC_POL_RESOURCE = Aliases(
+    [
+        "linux-svc-policy",
+        "linux-svc-policies",
+        "lsvc-pol",
+    ],
+    "linux-svc-policy",
+    "linux-svc-policies",
+    kind=POL_KIND,
+)
+ROLES_RESOURCE = Aliases(["roles", "role"], "role", "roles")
+CLUSTERROLES_RESOURCE = Aliases(
+    ["clusterroles", "clusterrole"], "clusterrole", "clusterroles"
+)
+ROLEBINDING_RESOURCE = Aliases(
+    ["rolebinding", "rolebindings", "rb"], "rolebinding", "rolebindings"
+)
+CLUSTERROLE_BINDING_RESOURCE = Aliases(
+    ["clusterrolebinding", "clusterrolebindings", "crb"],
+    "clusterrolebinding",
+    "clusterrolebindings",
+)
+
 POLICIES_RESOURCE = Aliases(
     [
         "policies",
@@ -179,6 +288,31 @@ POLICIES_RESOURCE = Aliases(
     ],
     "policy",
     "policies",
+    kind=POL_KIND,
+)
+CLUSTER_RULESET_RESOURCE = Aliases(
+    [
+        "cluster-ruleset",
+        "cluster-rulesets",
+        "cluster-rules",
+        "cluster-rule",
+        "cluster-rs",
+        "clus_rs",
+        "crs",
+    ],
+    "cluster-ruleset",
+    "cluster-rulesets",
+    kind=RULESET_KIND,
+)
+RULESETS_RESOURCE = Aliases(
+    [
+        "ruleset",
+        "rulesets",
+        "rs",
+    ],
+    "ruleset",
+    "rulesets",
+    kind=RULESET_KIND,
 )
 PROCESSES_RESOURCE = Aliases(
     [
@@ -190,6 +324,12 @@ PROCESSES_RESOURCE = Aliases(
     "process",
     "processes",
 )
+SAVED_QUERY_RESOURCE = Aliases(
+    ["saved-query", "saved-queries", "sq"],
+    "saved-query",
+    "saved-queries",
+    kind=SAVED_QUERY_KIND,
+)
 SOURCES_RESOURCE = Aliases(["source", "sources", "src"], "source", "sources")
 SPYDERTRACE_RESOURCE = Aliases(
     ["spydertrace", "spydertraces", "spyder", "trace", "traces"],
@@ -207,19 +347,42 @@ SPYDERTRACE_SUMMARY_RESOURCE = Aliases(
     "spydertrace-summary",
     "spydertrace-summaries",
 )
-SUPPRESSION_POLICY_RESOURCE = Aliases(
+REPLICASET_RESOURCE = Aliases(
+    ["replicaset", "replicasets", "rs", "replica"],
+    "replicaset",
+    "replicasets",
+)
+TRACE_SUPPRESSION_POLICY_RESOURCE = Aliases(
     [
-        "suppression-policy",
-        "suppression-policies",
-        "sp",
-        "s-pol",
+        "trace-suppression-policy",
+        "trace-suppression-policies",
+        "tsp",
         "trace-policy",
+        "trace-policies",
     ],
-    "suppression-policy",
-    "suppression-policies",
+    "trace-suppression-policy",
+    "trace-suppression-policies",
+    kind=POL_KIND,
+)
+TOP_DATA_RESOURCE = Aliases(
+    [
+        "top-data",
+        "top",
+    ],
+    "top-data",
+    "top-data",
 )
 UID_LIST_RESOURCE = Aliases(
-    ["uid-list", "uid-lists", "uid", "uids-list"], "uid-list", "uid-lists"
+    ["uid-list", "uid-lists", "uid", "uids-list"],
+    "uid-list",
+    "uid-lists",
+    kind=UID_LIST_KIND,
+)
+
+REPORT_RESOURCE = Aliases(
+    ["report", "reports", "rep"],
+    "report",
+    "reports",
 )
 
 SECRETS_ALIAS = Aliases(["secret", "secrets", "sec", "s"], "secret", "secrets")
@@ -244,35 +407,36 @@ def get_plural_name_from_alias(alias: str):
 
 
 DEL_RESOURCES: List[str] = [
+    CLUSTER_POLICY_RESOURCE.name,
+    CLUSTER_RULESET_RESOURCE.name,
+    CONTAINER_POL_RESOURCE.name,
+    LINUX_SVC_POL_RESOURCE.name,
+    NOTIFICATION_CONFIGS_RESOURCE.name,
+    NOTIFICATION_TARGETS_RESOURCE.name,
     POLICIES_RESOURCE.name,
-    SUPPRESSION_POLICY_RESOURCE.name,
+    RULESETS_RESOURCE.name,
+    SAVED_QUERY_RESOURCE.name,
+    TRACE_SUPPRESSION_POLICY_RESOURCE.name,
 ]
 DESC_RESOURCES: List[str] = [
     POLICIES_RESOURCE.name,
 ]
-GET_RESOURCES: List[str] = [
-    AGENT_RESOURCE.name_plural,
-    CLUSTERS_RESOURCE.name_plural,
-    CONNECTIONS_RESOURCE.name_plural,
-    CONNECTION_BUN_RESOURCE.name_plural,
-    CONTAINER_RESOURCE.name_plural,
-    DEPLOYMENTS_RESOURCE.name_plural,
-    DEVIATIONS_RESOURCE.name_plural,
-    FINGERPRINTS_RESOURCE.name_plural,
-    MACHINES_RESOURCE.name_plural,
-    NAMESPACES_RESOURCE.name_plural,
-    NODES_RESOURCE.name_plural,
-    OPSFLAGS_RESOURCE.name_plural,
-    PODS_RESOURCE.name_plural,
-    POLICIES_RESOURCE.name_plural,
-    PROCESSES_RESOURCE.name_plural,
-    REDFLAGS_RESOURCE.name_plural,
-    SOURCES_RESOURCE.name_plural,
-    # SPYDERTRACE_SUMMARY_RESOURCE.name_plural,
-    SUPPRESSION_POLICY_RESOURCE.name_plural,
-    CONTAINER_RESOURCE.name_plural,
-    SPYDERTRACE_RESOURCE.name_plural,
+EDIT_RESOURCES: List[str] = [
+    CLUSTER_POLICY_RESOURCE.name,
+    CLUSTER_RULESET_RESOURCE.name,
+    CONTAINER_POL_RESOURCE.name,
+    LINUX_SVC_POL_RESOURCE.name,
+    NOTIFICATION_CONFIGS_RESOURCE.name,
+    NOTIFICATION_TARGETS_RESOURCE.name,
+    POLICIES_RESOURCE.name,
+    RULESETS_RESOURCE.name,
+    TRACE_SUPPRESSION_POLICY_RESOURCE.name,
 ]
+EXPORT_RESOURCES: List[str] = [
+    POLICIES_RESOURCE.name_plural,
+    NOTIFICATION_CONFIGS_RESOURCE.name_plural,
+]
+
 LOGS_RESOURCES: List[str] = [POLICIES_RESOURCE.name]
 VAL_RESOURCES: List[str] = [
     BASELINES_RESOURCE.name,
@@ -286,13 +450,18 @@ RESOURCES_WITH_SCHEMAS = [
     FINGERPRINTS_RESOURCE.name,
     FINGERPRINT_GROUP_RESOURCE.name,
     POLICIES_RESOURCE.name,
+    CLUSTER_POLICY_RESOURCE.name,
     SECRETS_ALIAS.name,
-    SUPPRESSION_POLICY_RESOURCE.name,
+    TRACE_SUPPRESSION_POLICY_RESOURCE.name,
     UID_LIST_RESOURCE.name,
 ]
 
 CMD_ORG_FIELD = "org"
 
+SUB_EPILOG = (
+    'Use "spyctl <command> --help" for more information about a given command.'
+)
+
 
 def tmp_context_options(function):
     function = click.option(f"--{CMD_ORG_FIELD}", hidden=True)(function)
@@ -336,15 +505,15 @@ class DescribeResourcesParam(click.ParamType):
         ]
 
 
-class GetResourcesParam(click.ParamType):
-    name = "get_resources"
+class EditResourcesParam(click.ParamType):
+    name = "edit_resources"
 
     def shell_complete(
         self, ctx: click.Context, param: click.Parameter, incomplete: str
     ) -> List["CompletionItem"]:
         return [
             CompletionItem(resrc_name)
-            for resrc_name in GET_RESOURCES
+            for resrc_name in EDIT_RESOURCES
             if resrc_name.startswith(incomplete)
         ]
 
@@ -375,19 +544,6 @@ class SuppressionPolTypeParam(click.ParamType):
         ]
 
 
-class LabelParam(click.ParamType):
-    def convert(
-        self,
-        value: Any,
-        param: Optional[click.Parameter],
-        ctx: Optional[click.Context],
-    ) -> Any:
-        rv = label_input_to_dict()
-        if rv is None:
-            self.fail("Invalid label input", param, ctx)
-        return value
-
-
 class ListParam(click.ParamType):
     def convert(
         self,
@@ -398,12 +554,27 @@ class ListParam(click.ParamType):
         if "," in value:
             rv = value.split(",")
         else:
-            rv = value.split(" ")
+            rv = [value]
         for i, v in enumerate(rv):
             rv[i] = v.strip(" ")
         return rv
 
 
+class DictParam(click.ParamType):
+    def convert(
+        self,
+        value: Any,
+        param: Optional[click.Parameter],
+        ctx: Optional[click.Context],
+    ) -> Dict:
+        rv_dict = {}
+        args_list: List[str] = value.split(",")
+        for v in args_list:
+            key, val = v.strip().split("=")
+            rv_dict[key.strip()] = val.strip()
+        return rv_dict
+
+
 class ListDictParam(click.ParamType):
     def convert(
         self,
@@ -426,6 +597,19 @@ class ListDictParam(click.ParamType):
         return rv
 
 
+class ExportResourcesParam(click.ParamType):
+    name = "export_resources"
+
+    def shell_complete(
+        self, ctx: click.Context, param: click.Parameter, incomplete: str
+    ) -> List["CompletionItem"]:
+        return [
+            CompletionItem(resrc_name)
+            for resrc_name in EXPORT_RESOURCES
+            if resrc_name.startswith(incomplete)
+        ]
+
+
 class FileList(click.File):
     def convert(
         self,
@@ -481,16 +665,22 @@ class FileList(click.File):
 SCHEMA_FIELD = "schema"
 
 # Spyderbat Event Schema Prefix'
-EVENT_AGENT_METRICS_PREFIX = "event_agentmetrics"
-EVENT_AUDIT_PREFIX = "event_audit"
+EVENT_METRICS_PREFIX = "event_metric"
 EVENT_OPSFLAG_PREFIX = "event_opsflag"
 EVENT_REDFLAG_PREFIX = "event_redflag"
+EVENT_DEVIATION_PREFIX = "event_deviation"
+EVENT_LOG_PREFIX = "event_log"
+EVENT_TOP_DATA_PREFIX = "event_top_data"
 
-EVENT_AUDIT_SUBTYPE_MAP = {
+EVENT_METRIC_SUBTYPE_MAP = {
+    "agent": "agent",
+    "machine": "machine",
+}
+
+EVENT_LOG_SUBTYPE_MAP = {
     "deviation": "guardian_deviation",
     "action": "guardian_action",
-    "redflag": "guardian_redflag",
-    "opsflag": "guardian_opsflag",
+    "redflag": "guardian_flag",
 }
 
 # Spyderbat Model Schema Prefix'
@@ -502,11 +692,17 @@ MODEL_CONTAINER_PREFIX = "model_container"
 MODEL_DEPLOYMENT_PREFIX = "model_k8s_deployment"
 MODEL_FINGERPRINT_PREFIX = "model_fingerprint"
 MODEL_MACHINE_PREFIX = "model_machine"
+MODEL_NAMESPACE_PREFIX = "model_k8s_namespace"
 MODEL_NODE_PREFIX = "model_k8s_node"
 MODEL_POD_PREFIX = "model_k8s_pod"
+MODEL_REPLICASET_PREFIX = "model_k8s_replicaset"
+MODEL_K8S_ROLE_PREFIX = "model_k8s_role:"
+MODEL_ROLEBINDING_PREFIX = "model_k8s_rolebinding:"
+MODEL_CLUSTERROLE_BINDING_PREFIX = "model_k8s_clusterrolebinding"
+MODEL_K8S_CLUSTERROLE_PREFIX = "model_k8s_clusterrole:"
 MODEL_PROCESS_PREFIX = "model_process"
 MODEL_SPYDERTRACE_PREFIX = "model_spydertrace"
-
+MODEL_DAEMONSET_PREFIX = "model_k8s_daemonset"
 MODEL_FINGERPRINT_SUBTYPE_MAP = {
     "container": "container",
     "linux-service": "linux_svc",
@@ -519,15 +715,7 @@ DATATYPE_FINGERPRINTS = "fingerprints"
 DATATYPE_K8S = "k8s"
 DATATYPE_REDFLAGS = "redflags"
 DATATYPE_SPYDERGRAPH = "spydergraph"
-
-# Resource Kinds
-POL_KIND = "SpyderbatPolicy"
-SUP_POL_KIND_ALIAS = "SuppressionPolicy"
-BASELINE_KIND = "SpyderbatBaseline"
-FPRINT_KIND = "SpyderbatFingerprint"
-FPRINT_GROUP_KIND = "FingerprintGroup"
-UID_LIST_KIND = "UidList"
-DEVIATION_KIND = "GuardianDeviation"
+DATATYPE_HTOP = "htop"
 
 # CONFIG Kinds
 CONFIG_KIND = "Config"
@@ -621,6 +809,7 @@ TEMPLATE_FIELD = "template"
 
 
 # Selectors
+CLUS_SELECTOR_FIELD = "clusterSelector"
 CONT_SELECTOR_FIELD = "containerSelector"
 DNS_SELECTOR_FIELD = "dnsSelector"
 MACHINE_SELECTOR_FIELD = "machineSelector"
@@ -631,8 +820,19 @@ SVC_SELECTOR_FIELD = "serviceSelector"
 TRACE_SELECTOR_FIELD = "traceSelector"
 USER_SELECTOR_FIELD = "userSelector"
 MATCH_LABELS_FIELD = "matchLabels"
+MATCH_EXPRESSIONS_FIELD = "matchExpressions"
+MATCH_FIELDS_FIELD = "matchFields"
+MATCH_FIELDS_EXPRESSIONS_FIELD = "matchFieldsExpressions"
+KEY_FIELD = "key"
+OPERATOR_FIELD = "operator"
+IN_OPERATOR = "In"
+NOT_IN_OPERATOR = "NotIn"
+EXISTS_OPERATOR = "Exists"
+DOES_NOT_EXIST_OPERATOR = "DoesNotExist"
+VALUES_FIELD = "values"
 # Machine Selector Fields
 HOSTNAME_FIELD = "hostname"
+MACHINE_UID_FIELD = "machineUID"
 # Container Selector Fields
 IMAGE_FIELD = "image"
 IMAGEID_FIELD = "imageID"
@@ -657,10 +857,11 @@ BE_POL_UID_FIELD = (
 )
 POL_TYPE_CONT = "container"
 POL_TYPE_SVC = "linux-service"
+POL_TYPE_CLUS = "cluster"
 POL_TYPE_TRACE = "trace"
 SUPPRESSION_POL_TYPES = [POL_TYPE_TRACE]
-GUARDIAN_POL_TYPES = [POL_TYPE_CONT, POL_TYPE_SVC]
-POL_TYPES = [POL_TYPE_SVC, POL_TYPE_CONT, POL_TYPE_TRACE]
+GUARDIAN_POL_TYPES = [POL_TYPE_CONT, POL_TYPE_SVC, POL_TYPE_CLUS]
+POL_TYPES = [POL_TYPE_SVC, POL_TYPE_CONT, POL_TYPE_TRACE, POL_TYPE_CLUS]
 POL_MODE_ENFORCE = "enforce"
 POL_MODE_AUDIT = "audit"
 POL_MODES = [POL_MODE_ENFORCE, POL_MODE_AUDIT]
@@ -702,10 +903,16 @@ METADATA_TAGS_FIELD = "tags"
 METADATA_TYPE_FIELD = "type"
 METADATA_UID_FIELD = "uid"
 METADATA_CREATE_TIME = "creationTimestamp"
+METADATA_CREATED_BY = "createdBy"
+METADATA_LAST_UPDATE_TIME = "lastUpdatedTimestamp"
+METADATA_LAST_UPDATED_BY = "lastUpdatedBy"
+METADATA_LAST_USED = "lastUsedTimestamp"
+METADATA_VERSION_FIELD = "version"
 METADATA_NAMESPACE_FIELD = "namespace"
 METADATA_S_CHECKSUM_FIELD = "selectorHash"
 METADATA_START_TIME_FIELD = "startTime"
 METADATA_END_TIME_FIELD = "endTime"
+METADATA_SCOPES_FIELD = "scopes"
 NET_POLICY_FIELD = "networkPolicy"
 PROC_POLICY_FIELD = "processPolicy"
 FIRST_TIMESTAMP_FIELD = "firstTimestamp"
@@ -742,6 +949,33 @@ SUP_POL_SELECTOR_FIELDS = [
 ]
 TRACE_SUMMARY_FIELD = "trace_summary"
 
+# Policy Rulesets
+RULESET_TYPE_CLUS = "cluster"
+RULESET_TYPE_CONT = "container"
+RULESET_TYPES = [RULESET_TYPE_CLUS, RULESET_TYPE_CONT]
+RULESETS_FIELD = "rulesets"
+RULES_FIELD = "rules"
+RULES_TYPE_CONTAINER = "container"
+RULE_MATCHES_FIELD = "matches"
+CLUSTER_RULESET_RULE_TYPES = [RULES_TYPE_CONTAINER]
+RULE_VERB_ALLOW = "allow"
+RULE_VERB_DENY = "deny"
+RULE_VERBS = [RULE_VERB_ALLOW, RULE_VERB_DENY]
+RULE_VERB_FIELD = "verb"
+RULE_VALUES_FIELD = "values"
+RULE_TARGET_FIELD = "target"
+CONTAINER_RULE_TARGETS = [
+    "container::image",
+    "container::imageID",
+    "container::containerName",
+    "container::containerID",
+]
+PROCESS_RULE_TARGETS = [
+    "process::exe",
+    "process::name",
+    "process::euser",
+]
+
 NOT_AVAILABLE = "N/A"
 # Fingerprint Groups
 FPRINT_GRP_FINGERPRINTS_FIELD = "fingerprints"
@@ -839,6 +1073,7 @@ AGENT_STATUS = "status"
 AGENT_HOSTNAME = "hostname"
 AGENT_ID = "id"
 AGENT_BAT_STATUSES = "bat_statuses"
+AGENT_CLUSTER_NAME = "cluster_name"
 
 # Spydertraces
 BE_TRIGGER_NAME = "trigger_short_name"
@@ -866,18 +1101,136 @@ PROCESSES_FIELD = "processes"
 PROTO_FIELD = "protocol"
 TO_FIELD = "to"
 
+
+# Notifications
+NOTIF_TYPE_ALL = "all"
+NOTIF_TYPE_OBJECT = "object"
+NOTIF_TYPE_METRICS = "metrics"
+NOTIF_TYPE_DASHBOARD = "dashboard"
+NOTIF_TYPES = [
+    NOTIF_TYPE_ALL,
+    NOTIF_TYPE_OBJECT,
+    NOTIF_TYPE_METRICS,
+    NOTIF_TYPE_DASHBOARD,
+]
+NOTIF_TYPE_FIELD = "type"
+NOTIF_TMPL_TYPES = ["agent-health", "security", "operations", "guardian"]
+NOTIF_TMPL_MAP = {
+    "agent-health": "agent_health",
+    "security": "security",
+    "operations": "operations",
+    "guardian": "guardian",
+}
+DST_TYPE_ORG = "org_uid"
+DST_TYPE_EMAIL = "emails"
+DST_TYPE_SLACK = "slack"
+DST_TYPE_SNS = "sns"
+DST_TYPE_USERS = "users"
+DST_TYPE_WEBHOOK = "webhook"
+DST_NAME_EMAIL = "Email"
+DST_NAME_SLACK = "Slack"
+DST_NAME_SNS = "SNS"
+DST_NAME_WEBHOOK = "Webhook"
+DST_TYPES = [
+    DST_TYPE_EMAIL,
+    DST_TYPE_SLACK,
+    DST_TYPE_SNS,
+    DST_TYPE_WEBHOOK,
+]
+DST_NAMES = [
+    DST_NAME_EMAIL,
+    DST_NAME_SLACK,
+    DST_NAME_SNS,
+    DST_NAME_WEBHOOK,
+]
+DST_NAME_TO_TYPE = {
+    DST_NAME_EMAIL: DST_TYPE_EMAIL,
+    DST_NAME_SLACK: DST_TYPE_SLACK,
+    DST_NAME_SNS: DST_TYPE_SNS,
+    DST_NAME_WEBHOOK: DST_TYPE_WEBHOOK,
+}
+DST_TYPE_TO_NAME = {
+    DST_TYPE_EMAIL: DST_NAME_EMAIL,
+    DST_TYPE_SLACK: DST_NAME_SLACK,
+    DST_TYPE_SNS: DST_NAME_SNS,
+    DST_TYPE_WEBHOOK: DST_NAME_WEBHOOK,
+}
+DST_TYPE_TO_DESC = {
+    DST_TYPE_EMAIL: "A list of email addresses to send notifications to.",
+    DST_TYPE_SLACK: "A Slack hook URL to send notifications to.",
+    DST_TYPE_SNS: "An AWS sns endpoint to send notifications to.",
+    DST_TYPE_WEBHOOK: "A generic webhook URL to send notifications to.",
+}
+ROUTES_FIELD = "routes"
+TARGETS_FIELD = "targets"
+DST_DESCRIPTION = "description"
+DST_DATA = "data"
+DST_SNS_TOPIC_ARN = "sns_topic_arn"
+DST_SNS_CROSS_ACCOUNT_ROLE = "cross_account_iam_role"
+DST_WEBHOOK_URL = "url"
+DST_WEBHOOK_TLS_VAL = "no_tls_validation"
+DST_SLACK_URL = "url"
+ROUTE_TARGETS = "targets"
+ROUTE_DESTINATION = "destination"
+ROUTE_DATA = "data"
+ROUTE_DATA_ANA_SETTINGS = "analyticsSettings"
+ROUTE_DESCRIPTION = "description"
+ROUTE_EXPR = "expr"
+TGT_DESCRIPTION_FIELD = "description"
+TMPL_DESCRIPTION_FIELD = "description"
+TMPL_CONFIG_VALUES_FIELD = "configValues"
+
+NOTIF_ADDITIONAL_FIELDS = "additionalFields"
+NOTIF_DST_TGTS = "targets"
+NOTIF_DATA_FIELD = "data"
+NOTIF_CONDITION_FIELD = "condition"
+NOTIF_COOLDOWN_FIELD = "cooldown"
+NOTIF_COOLDOWN_BY_FIELD_FIELD = "byField"
+NOTIF_COOLDOWN_SECONDS_FIELD = "forSeconds"
+NOTIF_FOR_DURATION_FIELD = "forDuration"
+NOTIF_SETTINGS_FIELD = "analyticsConfiguration"
+NOTIF_NAME_FIELD = "name"
+NOTIF_INTERVAL_FIELD = "interval"
+NOTIF_TITLE_FIELD = "title"
+NOTIF_MESSAGE_FIELD = "message"
+NOTIF_ICON_FIELD = "icon"
+NOTIF_NOTIFY_FIELD = "notify"
+NOTIF_CREATE_TIME = "createTime"
+NOTIF_LAST_UPDATED = "lastUpdated"
+NOTIF_TEMPLATE_FIELD = "template"
+NOTIF_TARGET_FIELD = "target"
+NOTIF_DEFAULT_SCHEMA = "schemaType"
+NOTIF_SUB_SCHEMA = "subSchema"
+ANA_NOTIF_TYPE_AGENT_HEALTH = "agent_health"
+ANA_NOTIF_TYPE_CUSTOM = "custom"
+
+# Saved Query
+QUERY_SCHEMA_FIELD = "schema"
+QUERY_FIELD = "query"
+QUERY_DESCRIPTION_FIELD = "description"
+
+
+def get_dst_type(name):
+    return DST_NAME_TO_TYPE[name]
+
+
+def get_dst_name(type):
+    return DST_NAME_TO_TYPE[type]
+
+
 # Flags
 FLAG_CLASS = "class"
 
 # Output
 OUTPUT_YAML = "yaml"
 OUTPUT_JSON = "json"
+OUTPUT_NDJSON = "ndjson"
 OUTPUT_DEFAULT = "default"
 OUTPUT_RAW = "raw"
 OUTPUT_WIDE = "wide"
 # used internally when updating objects directly via the API
 OUTPUT_API = "api"
-OUTPUT_CHOICES = [OUTPUT_YAML, OUTPUT_JSON, OUTPUT_DEFAULT]
+OUTPUT_CHOICES = [OUTPUT_YAML, OUTPUT_JSON, OUTPUT_NDJSON, OUTPUT_DEFAULT]
 OUTPUT_DEST_DEFAULT = "default"  # stdout
 OUTPUT_DEST_FILE = "file"
 OUTPUT_DEST_API = "api"
@@ -889,6 +1242,7 @@ CLUSTER_OPTION = "cluster"
 NAMESPACE_OPTION = "namespace"
 
 # deviations
+DEVIATION_FIELD = "deviation"
 DEVIATION_DESCRIPTION = "deviationDescription"
 
 # Templates
@@ -1005,7 +1359,7 @@ def walk_up_tree(
     return rv
 
 
-def load_file(path: Path) -> Dict:
+def load_file(path: Path):
     try:
         with path.open("r") as f:
             try:
@@ -1112,7 +1466,7 @@ class CustomGroup(click.Group):
                         formatter.write_dl(rows, col_spacing=4)
 
 
-class CustomSubGroup(click.Group):
+class CustomSubGroup(ClickAliasedGroup):
     def group(self, *args, **kwargs):
         """Behaves the same as `click.Group.group()` except if passed
         a list of names, all after the first will be aliases for the first.
@@ -1181,35 +1535,39 @@ class CustomSubGroup(click.Group):
             formatter.write_paragraph()
             formatter.write_text(epilog)
 
-    def format_commands(
-        self, ctx: click.Context, formatter: click.HelpFormatter
-    ) -> None:
-        commands = []
-        for subcommand in self.list_commands(ctx):
-            cmd = self.get_command(ctx, subcommand)
-            # What is this, the tool lied about a command.  Ignore it
-            if cmd is None:
-                continue
-            if cmd.hidden:
-                continue
+    # def format_commands(
+    #     self, ctx: click.Context, formatter: click.HelpFormatter
+    # ) -> None:
+    #     commands = []
+    #     for subcommand in self.list_commands(ctx):
+    #         cmd = self.get_command(ctx, subcommand)
+    #         # What is this, the tool lied about a command.  Ignore it
+    #         if cmd is None:
+    #             continue
+    #         if cmd.hidden:
+    #             continue
 
-            commands.append((subcommand, cmd))
+    #         commands.append((subcommand, cmd))
 
-        # allow for 3 times the default spacing
-        if len(commands):
-            limit = formatter.width - 6 - max(len(cmd[0]) for cmd in commands)
+    #     # allow for 3 times the default spacing
+    #     if len(commands):
+    #         limit = formatter.width - 6 - max(len(cmd[0]) for cmd in commands)
 
-            rows = []
-            for subcommand, cmd in commands:
-                help = cmd.get_short_help_str(limit)
-                rows.append((subcommand, help))
+    #         rows = []
+    #         for subcommand, cmd in commands:
+    #             help = cmd.get_short_help_str(limit)
+    #             rows.append((subcommand, help))
 
-            if rows:
-                with formatter.section("Available Commands"):
-                    formatter.write_dl(rows, col_spacing=4)
+    #         if rows:
+    #             with formatter.section("Available Commands"):
+    #                 formatter.write_dl(rows, col_spacing=4)
 
 
 class CustomCommand(click.Command):
+    def __init__(self, *args, **kwargs) -> None:
+        self.aliases = kwargs.pop("aliases", [])
+        super().__init__(*args, **kwargs)
+
     def format_help(
         self, ctx: click.Context, formatter: click.HelpFormatter
     ) -> None:
@@ -1260,6 +1618,7 @@ class ArgumentParametersCommand(CustomCommand):
         self.unspecific = True
 
     def parse_args(self, ctx, args: List[str]) -> List[str]:
+        self.__reset_params()
         args_cpy = args.copy()
         parser = self.make_parser(ctx)
         parser.ignore_unknown_options = True
@@ -1279,7 +1638,6 @@ class ArgumentParametersCommand(CustomCommand):
                     if argument_value == value_option:
                         self.argument_value = str(value_option)
                         for arg_maker in obj["args"]:
-                            # single use, parse args twice will make dupes
                             arg_maker(self)
                         break
         return super().parse_args(ctx, args)
@@ -1321,6 +1679,9 @@ class ArgumentParametersCommand(CustomCommand):
                 with formatter.section(f"Options for {self.argument_value}"):
                     formatter.write_dl(specif_opts)
 
+    def __reset_params(self):
+        self.params = self.params[: self.base_param_count]
+
 
 class MutuallyExclusiveOption(click.Option):
     def __init__(self, *args, **kwargs):
@@ -1444,7 +1805,9 @@ def api_log(*args, **kwargs):
         err_exit("Broken Pipe")
 
 
-def time_inp(time_str: str, cap_one_day=False) -> int:
+def time_inp(time_str: str, cap_one_day=False) -> Optional[int]:
+    if time_str is None:
+        return None
     past_seconds = 0
     epoch_time = None
     try:
@@ -1452,15 +1815,15 @@ def time_inp(time_str: str, cap_one_day=False) -> int:
             epoch_time = int(time_str)
         except ValueError:
             if time_str.endswith(("s", "sc")):
-                past_seconds = int(time_str.split("s")[0])
+                past_seconds = float(time_str.split("s")[0])
             elif time_str.endswith(("m", "mn")):
-                past_seconds = int(time_str.split("m")[0]) * 60
+                past_seconds = float(time_str.split("m")[0]) * 60
             elif time_str.endswith(("h", "hr")):
-                past_seconds = int(time_str.split("h")[0]) * 60 * 60
+                past_seconds = float(time_str.split("h")[0]) * 60 * 60
             elif time_str.endswith(("d", "dy")):
-                past_seconds = int(time_str.split("d")[0]) * 60 * 60 * 24
+                past_seconds = float(time_str.split("d")[0]) * 60 * 60 * 24
             elif time_str.endswith(("w", "wk")):
-                past_seconds = int(time_str.split("w")[0]) * 60 * 60 * 24 * 7
+                past_seconds = float(time_str.split("w")[0]) * 60 * 60 * 24 * 7
             else:
                 date = dateparser.parse(time_str)
                 date = date.replace(tzinfo=date.tzinfo or timezone.utc)
@@ -1655,7 +2018,7 @@ def make_uuid():
 
 
 def err_exit(message: str, exception: Exception = None):
-    if API_CALL:
+    if API_CALL or INTERACTIVE:
         raise Exception(f"{WARNING_COLOR}Error: {message}{COLOR_END}")
     sys.exit(f"{WARNING_COLOR}Error: {message}{COLOR_END}")
 
@@ -1806,7 +2169,8 @@ def to_timestamp(zulu_str):
 def epoch_to_zulu(epoch):
     try:
         return (
-            zulu.Zulu.fromtimestamp(epoch).format("YYYY-MM-ddTHH:mm:ss") + "Z"
+            zulu.Zulu.fromtimestamp(epoch).format("YYYY-MM-ddTHH:mm:ss")
+            + " UTC"
         )
     except Exception:
         return epoch
@@ -1823,6 +2187,12 @@ def get_metadata_name(resource: Dict) -> Optional[str]:
     return name
 
 
+def get_metadata_type(resource: Dict) -> Optional[str]:
+    metadata = resource.get(METADATA_FIELD, {})
+    type = metadata.get(METADATA_TYPE_FIELD)
+    return type
+
+
 def slugify(value, allow_unicode=False):
     """
     Taken from https://github.com/django/django/blob/master/django/utils/text.py  # noqa E501
@@ -1915,6 +2285,13 @@ def set_api_call():
     disable_colorization()
 
 
+def set_interactive():
+    global INTERACTIVE, USE_LOG_VARS
+    INTERACTIVE = True
+    USE_LOG_VARS = True
+    disable_colorization()
+
+
 def set_debug():
     global DEBUG
     DEBUG = True
@@ -1965,3 +2342,104 @@ def calc_age(time_float: float):
     else:
         age = f"{age_delta.seconds//60}m"
         return age
+
+
+TGT_NAME_VALID_SYMBOLS = ["-", "_"]
+NOTIF_NAME_VALID_SYMBOLS = ["-", "_"]
+
+TGT_NAME_ERROR_MSG = (
+    "Target name must contain only letters, numbers, and"
+    f" {TGT_NAME_VALID_SYMBOLS}. It must also be less than 64"
+    " characters."
+)
+
+NOTIF_CONF_NAME_ERROR_MSG = "Name must be less than 64 characters."
+
+
+def is_valid_tgt_name(input_string):
+    pattern = r"^[a-zA-Z0-9\-_]+$"
+
+    if re.match(pattern, input_string) and len(input_string) <= 64:
+        return True
+    else:
+        return False
+
+
+def is_valid_notification_name(input_string) -> str:
+    if len(input_string) <= 64:
+        return True
+    return False
+
+
+def valid_notification_name(input_string) -> str:
+    pattern = r"^[a-zA-Z0-9\-_]+$"
+    if re.match(pattern, input_string) and len(input_string) <= 64:
+        return input_string
+    raise click.UsageError(
+        "Notification name must contain only letters, numbers, and"
+        f" {NOTIF_NAME_VALID_SYMBOLS}. It must also be less than 64"
+        " characters."
+    )
+
+
+def valid_schema(input_string) -> str:
+    if " " in input_string:
+        raise click.UsageError("Schemas may not have spaces.")
+    return input_string
+
+
+def is_valid_email(email):
+    # Define a regular expression pattern for a valid email address
+    pattern = r"^[\w\.-]+@[\w\.-]+\.\w+$"
+
+    # Use the re.match function to check if the email matches the pattern
+    if re.match(pattern, email):
+        return True
+    else:
+        return False
+
+
+def is_valid_url(url):
+    try:
+        result = urlparse(url)
+        return all(
+            [result.scheme, result.netloc]
+        )  # Check if both scheme and network location are present
+    except ValueError:
+        return False
+
+
+SLACK_AND_ALTERNATIVES = [
+    "https://hooks.slack.com/services/",
+    "https://chat.eencloud.com",
+]
+
+
+def is_valid_slack_url(url: str):
+    try:
+        result = urlparse(url)
+        if not all(
+            [result.scheme, result.netloc]
+        ):  # Check if both scheme and network location are present
+            return False
+        if not any(url.startswith(s) for s in SLACK_AND_ALTERNATIVES):
+            return False
+        return True
+    except ValueError:
+        return False
+
+
+def encode_int(x, length=4):
+    b = int(x).to_bytes(length, byteorder="big")
+    return b64url(b).decode("ascii").strip("=")
+
+
+def build_ctx() -> str:
+    return f"{encode_int(time.time())}.{make_uuid()[:5]}"
+
+
+def is_guardian_obj(obj: Dict):
+    kind = obj.get(KIND_FIELD)
+    type = obj.get(METADATA_FIELD, {}).get(METADATA_TYPE_FIELD)
+    if kind in [BASELINE_KIND, POL_KIND] and type in GUARDIAN_POL_TYPES:
+        return True
