@@ -1,4 +1,9 @@
 import json
+from pathlib import Path
+
+import yaml
+
+MOCK_RESOURCES_DIR = Path(__file__).parent / "mock_resources"
 
 
 # Mocking API Functionality
@@ -76,7 +81,7 @@ def mock_get_deployments(
     clusters,
     time,
     pipeline=None,
-    limit_mem: bool = False,
+    limit_mem: bool = True,
     disable_pbar_on_first: bool = False,
 ):
     mock_deployment = {
@@ -245,7 +250,7 @@ def mock_get_nodes(
     clusters,
     time,
     pipeline=None,
-    limit_mem: bool = False,
+    limit_mem: bool = True,
     disable_pbar_on_first: bool = False,
 ):
     mock_node = {
@@ -295,7 +300,7 @@ def mock_get_pods(
     clusters,
     time,
     pipeline=None,
-    limit_mem: bool = False,
+    limit_mem: bool = True,
     disable_pbar_on_first: bool = False,
 ):
     mock_pod = {
@@ -359,7 +364,7 @@ def mock_get_redflags(
     muids,
     time,
     pipeline=None,
-    limit_mem: bool = False,
+    limit_mem: bool = True,
     disable_pbar_on_first: bool = False,
 ):
     mock_redflag = {
@@ -420,7 +425,7 @@ def mock_get_opsflags(
     muids,
     time,
     pipeline=None,
-    limit_mem: bool = False,
+    limit_mem: bool = True,
     disable_pbar_on_first: bool = False,
 ):
     mock_opsflag = {
@@ -448,7 +453,7 @@ def mock_get_fingerprints(
     time,
     fprint_type=None,
     pipeline=None,
-    limit_mem: bool = False,
+    limit_mem: bool = True,
     disable_pbar_on_first: bool = False,
 ):
     mock_fingerprint = {
@@ -507,137 +512,89 @@ def mock_get_fingerprints(
         "version": 1689423128,
         "checksum": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
         "valid_to": 1689423288.2522442,
+        "covered_by_policy": True,
     }
     yield mock_fingerprint
 
 
-def mock_get_policies(api_url, api_key, org_uid, params=None):
-    mock_policy = {
+def mock_get_guardian_fingerprints(
+    api_url,
+    api_key,
+    org_uid,
+    sources,
+    time,
+    fprint_type=None,
+    unique=False,
+    limit_mem: bool = True,
+    disable_pbar_on_first: bool = False,
+    expr=None,
+    **filters,
+):
+    mock_fingerprint = {
+        "expire_at": 1689425999.999999,
+        "id": "fprint:linux-service:xxxxxxxxxxx:update-motd.service:xxxxxx:xxxx",
+        "cgroup": "systemd:/system.slice/update-motd.service",
+        "muid": "mach:xxxxxxxxxxx",
         "apiVersion": "spyderbat/v1",
-        "kind": "SpyderbatPolicy",
+        "kind": "SpyderbatFingerprint",
         "metadata": {
-            "name": "spyderbat-test",
-            "type": "container",
-            "latestTimestamp": 1672333396.3253918,
-            "creationTimestamp": 1672333396.3253918,
-            "uid": "1FZEoVkeS82aSI9jfLzm",
+            "name": "update-motd.service",
+            "id": "fprint:linux-service:xxxxxxxxxxx:update-motd.service:xxxxxx:xxxx",
+            "type": "linux-service",
+            "checksum": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
+            "org_uid": "spyderbatuid",
+            "muid": "mach:xxxxxxxxxxx",
+            "root": "proc:xxxxxxxxxxx:xxxxxx:9785",
+            "firstTimestamp": 1689423122.9874985,
+            "latestTimestamp": 1689423288.2522442,
+            "version": 1689423128,
         },
         "spec": {
-            "containerSelector": {
-                "image": "spyderbat-test",
-                "imageID": "sha256:6e2e1bce440ec41f53e849e56d5c6716ed7f1e1fa614d8dca2bbda49e5cde29e",
-            },
-            "podSelector": {
-                "matchLabels": {
-                    "app": "test",
-                    "env": "prod",
-                    "name": "test-web",
-                    "pod-template-hash": "8665ffd6c6",
-                    "tier": "frontend",
-                }
-            },
-            "mode": "audit",
-            "namespaceSelector": {
-                "matchLabels": {"kubernetes.io/metadata.name": "test"}
+            "serviceSelector": {
+                "cgroup": "systemd:/system.slice/update-motd.service"
             },
+            "machineSelector": {"hostname": "mock_machine"},
             "processPolicy": [
                 {
-                    "name": "python",
-                    "exe": ["/usr/local/bin/python3.7"],
-                    "id": "python_0",
+                    "name": "update-motd",
+                    "exe": ["/usr/bin/bash"],
+                    "id": "update-motd_0",
                     "euser": ["root"],
-                    "children": [
-                        {
-                            "name": "sh",
-                            "exe": ["/bin/dash"],
-                            "id": "sh_0",
-                            "children": [
-                                {
-                                    "name": "uname",
-                                    "exe": ["/bin/uname"],
-                                    "id": "uname_0",
-                                }
-                            ],
-                        }
-                    ],
-                },
-                {
-                    "name": "sh",
-                    "exe": ["/bin/dash"],
-                    "id": "sh_1",
-                    "euser": ["root"],
-                    "children": [
-                        {
-                            "name": "python",
-                            "exe": ["/usr/local/bin/python3.7"],
-                            "id": "python_1",
-                            "euser": ["web-svc"],
-                            "children": [
-                                {
-                                    "name": "sh",
-                                    "exe": ["/bin/dash"],
-                                    "id": "sh_2",
-                                    "children": [
-                                        {
-                                            "name": "uname",
-                                            "exe": ["/bin/uname"],
-                                            "id": "uname_1",
-                                        }
-                                    ],
-                                }
-                            ],
-                        }
-                    ],
-                },
+                }
             ],
             "networkPolicy": {
-                "ingress": [
-                    {
-                        "from": [{"ipBlock": {"cidr": "192.168.6.11/32"}}],
-                        "processes": ["python_0"],
-                        "ports": [{"protocol": "TCP", "port": 22}],
-                    },
-                    {
-                        "from": [{"ipBlock": {"cidr": "192.168.6.11/32"}}],
-                        "processes": ["python_1"],
-                        "ports": [{"protocol": "TCP", "port": 22}],
-                    },
-                ],
+                "ingress": [],
                 "egress": [
                     {
-                        "to": [{"dnsSelector": ["mongodb.local"]}],
-                        "processes": ["python_0"],
-                        "ports": [{"protocol": "TCP", "port": 27017}],
-                    },
-                    {
-                        "to": [{"ipBlock": {"cidr": "192.168.5.10/32"}}],
-                        "processes": ["python_0"],
-                        "ports": [{"protocol": "TCP", "port": 443}],
-                    },
-                    {
-                        "to": [{"dnsSelector": ["mongodb.local"]}],
-                        "processes": ["python_1"],
-                        "ports": [{"protocol": "TCP", "port": 27017}],
-                    },
-                    {
-                        "to": [
-                            {"ipBlock": {"cidr": "192.168.5.10/32"}},
-                            {"ipBlock": {"cidr": "192.168.5.11/32"}},
-                            {"ipBlock": {"cidr": "192.168.5.12/32"}},
-                            {"ipBlock": {"cidr": "192.168.5.13/32"}},
-                        ],
-                        "processes": ["python_1"],
-                        "ports": [{"protocol": "TCP", "port": 443}],
+                        "to": [{"ipBlock": {"cidr": "1.1.1.1/32"}}],
+                        "processes": ["yum_0"],
+                        "ports": [{"protocol": "TCP", "port": 80}],
                     },
                 ],
             },
-            "response": {
-                "default": [{"makeRedFlag": {"severity": "high"}}],
-                "actions": [],
-            },
         },
+        "root_puid": "proc:xxxxxxxxxxx:xxxxxx:9785",
+        "service_name": "update-motd.service",
+        "proc_fprint_len": 18,
+        "ingress_len": 0,
+        "egress_len": 3,
+        "schema": "model_fingerprint:linux_svc:1.0.0",
+        "status": "closed",
+        "time": 1689423288.2522442,
+        "valid_from": 1689423122.9874985,
+        "last_seen": 1689423126.8817828,
+        "version": 1689423128,
+        "checksum": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
+        "valid_to": 1689423288.2522442,
+        "covered_by_policy": True,
     }
-    return [mock_policy]
+    yield mock_fingerprint
+
+
+def mock_get_policies(api_url, api_key, org_uid, params=None, raw_data=False):
+    with open(MOCK_RESOURCES_DIR / "mock_policy.yaml", encoding="utf-8") as f:
+        resrc_data = yaml.load(f, Loader=yaml.SafeLoader)
+        return [resrc_data]
 
 
 def mock_post_new_policy(api_url, api_key, org_uid, data):
@@ -649,7 +606,7 @@ def mock_post_new_policy(api_url, api_key, org_uid, data):
     return MockResponse({"uid": "1FZEoVkeS82aSI9jfLzm"}, 200)
 
 
-def mock_put_policy_update(api_url, api_key, org_uid, pol_uid, data):
+def mock_put_policy_update(api_url, api_key, org_uid, data):
     class MockResponse:
         def __init__(self, json_data, status_code) -> None:
             self.status_code = status_code
