@@ -1,18 +1,20 @@
+import json
 import os
 import shutil
 from fnmatch import fnmatch
-from pathlib import Path
 from itertools import groupby
-import pytest
+from pathlib import Path
+from typing import Tuple
+from unittest import mock
 
+import pytest
 from click.testing import CliRunner
 
-from spyctl import spyctl
-from spyctl.spyctl_lib import time_inp
-from spyctl.config.configs import CURR_CONTEXT_NONE, set_testing
-import unittest.mock as mock
 import spyctl.tests.mock_functions as mock_func
-
+from spyctl import spyctl
+from spyctl.config.configs import CURR_CONTEXT_NONE, set_testing
+from spyctl.spyctl_lib import time_inp
+from spyctl.tests.backups import backup_secrets, restore_secrets
 
 API_KEY = os.environ.get("API_KEY")
 API_URL = os.environ.get("API_URL")
@@ -50,23 +52,23 @@ def test_help():
     assert response.exit_code == 0
     response = runner.invoke(spyctl.main, ["get", "redflags", "--help"])
     assert response.exit_code == 0
-    assert "Options for redflags:" in response.output
     response = runner.invoke(spyctl.main, ["config", "--help"])
     assert response.exit_code == 0
 
 
 @mock.patch(
-    "spyctl.commands.get.api.get_namespaces", mock_func.mock_get_namespaces
+    "spyctl.commands.get.namespaces.sq_api.get_namespaces",
+    mock_func.mock_get_namespaces,
 )
 def test_get_namespaces():
     get_resource("namespaces", TWOHOURS)
     get_resource("namespaces", TWOHOURS + OUTYML)
 
 
-TWOHOURS = ("-t", "2h")
-OUTYML = ("-o", "yaml")
-MINS_30 = ("-t", "30m")
-MINS_5 = ("-t", "5m")
+TWOHOURS = ["-t", "2h"]
+OUTYML = ["-o", "yaml"]
+MINS_30 = ["-t", "30m"]
+MINS_5 = ["-t", "5m"]
 
 
 def remove_timestamps(str_list):
@@ -121,57 +123,6 @@ resources = (
 )
 
 
-@mock.patch.multiple(
-    "spyctl.commands.get.api",
-    get_clusters=mock_func.mock_get_clusters,
-    get_deployments=mock_func.mock_get_deployments,
-    get_fingerprints=mock_func.mock_get_fingerprints,
-    get_namespaces=mock_func.mock_get_namespaces,
-    get_nodes=mock_func.mock_get_nodes,
-    get_opsflags=mock_func.mock_get_opsflags,
-    get_pods=mock_func.mock_get_pods,
-    get_policies=mock_func.mock_get_policies,
-    get_redflags=mock_func.mock_get_redflags,
-    get_sources=mock_func.mock_get_sources,
-)
-@mock.patch.multiple(
-    "spyctl.resources.deployments.api",
-    get_deployments=mock_func.mock_get_deployments,
-)
-@mock.patch.multiple(
-    "spyctl.resources.flags.api",
-    get_opsflags=mock_func.mock_get_opsflags,
-    get_redflags=mock_func.mock_get_redflags,
-)
-@mock.patch.multiple(
-    "spyctl.resources.namespaces.api",
-    get_namespaces=mock_func.mock_get_namespaces,
-)
-@mock.patch.multiple(
-    "spyctl.resources.nodes.api",
-    get_nodes=mock_func.mock_get_nodes,
-)
-@mock.patch.multiple(
-    "spyctl.resources.pods.api",
-    get_pods=mock_func.mock_get_pods,
-)
-@pytest.mark.parametrize("resource", resources)
-def test_get_resources(resource):
-    if resource not in ["opsflags", "fingerprints"]:
-        time_range = MINS_30
-    else:
-        time_range = TWOHOURS
-    output = OUTYML
-    table = get_resource(resource, time_range)
-    comparison_line, ids = process_table_response(table)
-    for name_or_id in ids:
-        single = get_resource(resource, time_range + (name_or_id,))
-        first_output = remove_timestamps(single.splitlines()[1].split())
-        assert first_output == comparison_line
-    time_range = MINS_5
-    get_resource(resource, time_range + output)
-
-
 def get_resource(resource, args=[], print_output=False):
     runner = CliRunner(mix_stderr=False)
     response = runner.invoke(spyctl.main, ["get", resource, *args])
@@ -181,6 +132,13 @@ def get_resource(resource, args=[], print_output=False):
     return response.stdout
 
 
+SPECIAL_GET_ARGS = {"fingerprints": ["--type", "linux-service"]}
+
+
+def get_args(resource: str) -> Tuple:
+    return SPECIAL_GET_ARGS.get(resource, [])
+
+
 resources_dir = Path(__file__).parent / "test_resources"
 
 
@@ -191,29 +149,46 @@ def test_create():
         ["create", "policy", "-f", resources_dir / "test_baseline.yaml"],
     )
     assert response.exit_code == 0
-    with open(resources_dir / "test_policy.yaml", "r") as f:
+    with open(resources_dir / "test_policy.yaml", "r", encoding="utf-8") as f:
         assert response.output.strip("\n") == f.read().strip("\n")
     response = runner.invoke(
         spyctl.main,
-        ["create", "baseline", "-f", resources_dir / "test_fprint_group.yaml"],
+        ["create", "policy", "-f", resources_dir / "test_fprint_group.yaml"],
     )
     assert response.exit_code == 0
-    with open(resources_dir / "test_baseline.yaml", "r") as f:
+    with open(
+        resources_dir / "test_created_policy.yaml", "r", encoding="utf-8"
+    ) as f:
         assert response.output.strip("\n") == f.read().strip("\n")
 
 
+@pytest.mark.skip(reason="For local testing. API testing is out of scope.")
+def test_create_api():
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "create",
+            "policy",
+            "-f",
+            resources_dir / "test_fingerprint_list.json",
+            "--api",
+        ],
+    )
+    assert response.exit_code == 0
+
+
 @mock.patch(
-    "spyctl.commands.get.api.get_policies", mock_func.mock_get_policies
+    "spyctl.commands.get.policies.pol_api.get_policies",
+    mock_func.mock_get_policies,
 )
 @mock.patch.multiple(
-    "spyctl.commands.apply.api",
-    get_policies=mock_func.mock_get_policies,
-    delete_policy=mock_func.mock_delete_policy,
+    "spyctl.commands.apply",
     post_new_policy=mock_func.mock_post_new_policy,
     put_policy_update=mock_func.mock_put_policy_update,
 )
 @mock.patch.multiple(
-    "spyctl.commands.delete.api",
+    "spyctl.commands.delete.policy",
     get_policies=mock_func.mock_get_policies,
     delete_policy=mock_func.mock_delete_policy,
 )
@@ -224,7 +199,7 @@ def test_update_policy():
     )
     assert response.exit_code == 0
     assert response.output.startswith(
-        "Successfully applied new policy with uid:"
+        "Successfully applied new container guardian policy with uid:"
     )
     response = runner.invoke(
         spyctl.main, ["get", "policies", "spyderbat-test"]
@@ -253,16 +228,15 @@ def test_update_policy():
 
 
 @mock.patch(
-    "spyctl.commands.get.api.get_policies", mock_func.mock_get_policies
+    "spyctl.commands.get.policies.pol_api.get_policies",
+    mock_func.mock_get_policies,
 )
 @mock.patch.multiple(
-    "spyctl.commands.apply.api",
-    get_policies=mock_func.mock_get_policies,
-    delete_policy=mock_func.mock_delete_policy,
+    "spyctl.commands.apply",
     post_new_policy=mock_func.mock_post_new_policy,
 )
 @mock.patch.multiple(
-    "spyctl.commands.delete.api",
+    "spyctl.commands.delete.policy",
     get_policies=mock_func.mock_get_policies,
     delete_policy=mock_func.mock_delete_policy,
 )
@@ -273,7 +247,7 @@ def test_apply_delete():
     )
     assert response.exit_code == 0
     assert response.output.startswith(
-        "Successfully applied new policy with uid:"
+        "Successfully applied new container guardian policy with uid:"
     )
     response = runner.invoke(
         spyctl.main, ["get", "policies", "spyderbat-test"]
@@ -300,6 +274,27 @@ def test_diff():
             "-y",
         ],
     )
+    print(response.output)
+    assert response.exit_code == 0
+    assert "+     - name: python3.7" in response.output
+
+
+@pytest.mark.skip(reason="For local testing. API testing is out of scope.")
+def test_diff_api():
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "diff",
+            "-f",
+            str(resources_dir) + "/test_baseline.yaml",
+            "-w",
+            str(resources_dir) + "/test_baseline_extra.yaml",
+            "-y",
+            "--api",
+        ],
+    )
+    print(response.output)
     assert response.exit_code == 0
     assert "+     - name: python3.7" in response.output
 
@@ -317,11 +312,109 @@ def test_merge():
             "-y",
         ],
     )
+    print(response.output)
     assert response.exit_code == 0
     assert "\n    - name: python3.7" in response.output
     assert "\n    - name: sh" in response.output
 
 
+@pytest.mark.skip(reason="For local testing. API testing is out of scope.")
+def test_merge_api():
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "merge",
+            "-f",
+            str(resources_dir) + "/test_baseline.yaml",
+            "-w",
+            str(resources_dir) + "/test_baseline_extra.yaml",
+            "-y",
+            "--api",
+        ],
+    )
+    print(response.output)
+    assert response.exit_code == 0
+    assert "'name': 'python3.7'" in response.output
+    assert "'name': 'sh'" in response.output
+
+
+@pytest.mark.skip(
+    reason="For local testing. API testing is out of scope. "
+    "UID lists must be updated as data expires."
+)
+def test_merge_api_uid_list():
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "merge",
+            "-f",
+            str(resources_dir) + "/pol_for_uid_list_merge_test.json",
+            "-w",
+            str(resources_dir) + "/test_uid_list.json",
+            "-y",
+            "--api",
+        ],
+    )
+    print(response.output)
+    assert response.exit_code == 0
+
+
+@pytest.mark.skip(
+    reason="For local testing. UID lists must be updated as data expires."
+)
+def test_merge_uid_list():
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "merge",
+            "-f",
+            str(resources_dir) + "/pol_for_uid_list_merge_test.json",
+            "-w",
+            str(resources_dir) + "/test_uid_list.json",
+            "-y",
+        ],
+    )
+    print(response.output)
+    assert response.exit_code == 0
+
+
+def test_validate(policy_path):
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "validate",
+            "-f",
+            policy_path,
+        ],
+    )
+    print(response.output)
+    assert response.exit_code == 0
+
+
+@pytest.mark.skip(reason="For local testing. API testing is out of scope.")
+def test_validate_api(policy_path):
+    runner = CliRunner()
+    response = runner.invoke(
+        spyctl.main,
+        [
+            "validate",
+            "-f",
+            policy_path,
+            "--api",
+        ],
+    )
+    assert response.exit_code == 0
+
+
+@pytest.fixture
+def policy_path():
+    return resources_dir / "test_policy.yaml"
+
+
 def env_setup() -> bool:
     # Tests that the proper environment variables exist in pyproject.toml
     if not API_KEY or API_KEY == "__NONE__":
@@ -388,6 +481,7 @@ def create_context(name):
             "-o",
             ORG,
             "-g",
+            "-u",
             name,
         ],
     )
@@ -437,6 +531,9 @@ def use_current_context():
 def setup_module():
     if not env_setup():
         raise SetupException("Check environment variables in pyproject.toml")
+    restore_secrets()  # In case the last test was cancelled
+    backup_secrets()
+
     global CURRENT_CONTEXT
     current_ctx = current_context()
     if current_ctx and current_ctx != CURR_CONTEXT_NONE:
@@ -462,3 +559,4 @@ def teardown_module():
     use_current_context()
     os.chdir(CURR_PATH)
     shutil.rmtree(WORKSPACE_PATH, ignore_errors=True)
+    restore_secrets()
