@@ -1,6 +1,6 @@
 import spyctl.schemas_v2 as schemas
 from fastapi import APIRouter, BackgroundTasks
-from pydantic import BaseModel, Field, Json
+from pydantic import BaseModel, Field
 
 import app.app_lib as app_lib
 import app.commands.validate as cmd_val
@@ -13,7 +13,7 @@ router = APIRouter(prefix="/api/v1")
 
 
 class ValidateHandlerInput(BaseModel):
-    object: Json[schemas.SpyderbatObject] = Field(
+    object: schemas.SpyderbatObject = Field(
         title="The object to be validated."
     )
 
@@ -30,6 +30,6 @@ def validate(
     i: ValidateHandlerInput, background_tasks: BackgroundTasks
 ) -> ValidateHandlerOutput:
     background_tasks.add_task(app_lib.flush_spyctl_log_messages)
-    validate_input = cmd_val.ValidateInput(i.object.dict(by_alias=True))
+    validate_input = cmd_val.ValidateInput(i.object.model_dump(by_alias=True))
     output = cmd_val.validate(validate_input)
     return ValidateHandlerOutput(invalid_message=output.invalid_message)
