@@ -1,11 +1,10 @@
-import json
 from dataclasses import dataclass
+from typing import Dict, List
 
 from spyctl.commands.merge import merge_resource
 
-import app.app_lib as app_lib
+from app import app_lib
 import app.exceptions as ex
-from typing import Dict, List
 
 # ------------------------------------------------------------------------------
 # Merge Object(s) into Object
@@ -39,5 +38,6 @@ def merge(i: MergeInput) -> MergeOutput:
     if not merged_object:
         msg = app_lib.flush_spyctl_log_messages()
         ex.internal_server_error(msg)
+    merged_object = merged_object[0]
     app_lib.flush_spyctl_log_messages()
-    return MergeOutput(json.dumps(merged_object.get_obj_data()))
+    return MergeOutput(merged_object.get_obj_data())
