@@ -1,12 +1,20 @@
-import logging
+from contextlib import asynccontextmanager
+
 import spyctl.config.configs as cfg
-from fastapi import FastAPI, Request, status
-from fastapi.exceptions import RequestValidationError
-from fastapi.responses import JSONResponse
+from fastapi import FastAPI
 
-from .api import create, diff, merge, validate
+from app import config
+from app.api import create, diff, merge, report, validate
 
-app = FastAPI()
+
+@asynccontextmanager
+async def lifespan(app: FastAPI):
+    # Load the config
+    await config.load_config()
+    yield
+
+
+app = FastAPI(lifespan=lifespan)
 
 
 @app.get("/alive")
@@ -30,9 +38,17 @@ async def root():
 #         content=content, status_code=status.HTTP_422_UNPROCESSABLE_ENTITY
 #     )
 
-
 app.include_router(create.router)
 app.include_router(diff.router)
 app.include_router(merge.router)
 app.include_router(validate.router)
+app.include_router(report.router)
 cfg.set_api_call()
+
+
+if __name__ == "__main__":
+    import os
+
+    import uvicorn
+
+    uvicorn.run(app, host="0.0.0.0", port=8000, log_level="debug")
